@@ -1,8 +1,9 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-adjustment.c: Adjustment object
  *
  * Copyright (C) 2008 OpenedHand
- * Copyright (c) 2009 Intel Corporation.
+ * Copyright (c) 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -49,7 +50,7 @@ struct _MxAdjustmentPrivate
 {
   /* Do not sanity-check values while constructing,
    * not all properties may be set yet. */
-  gboolean is_constructing : 1;
+  guint is_constructing : 1;
 
   gdouble  lower;
   gdouble  upper;
@@ -71,6 +72,7 @@ struct _MxAdjustmentPrivate
   ClutterTimeline *interpolation;
   gdouble          old_position;
   gdouble          new_position;
+  ClutterAlpha    *interpolate_alpha;
 
   /* For elasticity */
   gboolean      elastic;
@@ -101,16 +103,19 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0, };
 
-static gboolean mx_adjustment_set_lower          (MxAdjustment *adjustment,
-                                                  gdouble       lower);
-static gboolean mx_adjustment_set_upper          (MxAdjustment *adjustment,
-                                                  gdouble       upper);
-static gboolean mx_adjustment_set_step_increment (MxAdjustment *adjustment,
-                                                  gdouble       step);
-static gboolean mx_adjustment_set_page_increment (MxAdjustment *adjustment,
-                                                  gdouble       page);
-static gboolean mx_adjustment_set_page_size      (MxAdjustment *adjustment,
-                                                  gdouble       size);
+static gboolean _mx_adjustment_set_lower          (MxAdjustment *adjustment,
+                                                   gdouble       lower);
+static gboolean _mx_adjustment_set_upper          (MxAdjustment *adjustment,
+                                                   gdouble       upper);
+static gboolean _mx_adjustment_set_step_increment (MxAdjustment *adjustment,
+                                                   gdouble       step);
+static gboolean _mx_adjustment_set_page_increment (MxAdjustment *adjustment,
+                                                   gdouble       page);
+static gboolean _mx_adjustment_set_page_size      (MxAdjustment *adjustment,
+                                                   gdouble       size);
+static void mx_adjustment_clamp_page (MxAdjustment *adjustment,
+                                      gdouble       lower,
+                                      gdouble       upper);
 
 static void
 mx_adjustment_constructed (GObject *object)
@@ -270,6 +275,12 @@ mx_adjustment_dispose (GObject *object)
   mx_adjustment_remove_idle (&priv->page_size_source);
   mx_adjustment_remove_idle (&priv->changed_source);
 
+  if (priv->interpolate_alpha)
+    {
+      g_object_unref (priv->interpolate_alpha);
+      priv->interpolate_alpha = NULL;
+    }
+
   G_OBJECT_CLASS (mx_adjustment_parent_class)->dispose (object);
 }
 
@@ -380,13 +391,39 @@ mx_adjustment_init (MxAdjustment *self)
   self->priv->is_constructing = TRUE;
 }
 
+/**
+ * mx_adjustment_new:
+ *
+ * Create a new MxAdjustment
+ *
+ * Returns: a newly allocated MxAdjustment
+ */
 MxAdjustment *
-mx_adjustment_new (gdouble value,
-                   gdouble lower,
-                   gdouble upper,
-                   gdouble step_increment,
-                   gdouble page_increment,
-                   gdouble page_size)
+mx_adjustment_new (void)
+{
+  return g_object_new (MX_TYPE_ADJUSTMENT, NULL);
+}
+
+/**
+ * mx_adjustment_new_with_values:
+ * @value: A #gdouble
+ * @lower: A #gdouble
+ * @upper: A #gdouble
+ * @step_increment: A #gdouble
+ * @page_increment: A #gdouble
+ * @page_size: A #gdouble
+ *
+ * Create a new MxAdjustment with the properties set to the values specified.
+ *
+ * Returns: a newly allocated MxAdjustment
+ */
+MxAdjustment *
+mx_adjustment_new_with_values (gdouble value,
+                               gdouble lower,
+                               gdouble upper,
+                               gdouble step_increment,
+                               gdouble page_increment,
+                               gdouble page_size)
 {
   return g_object_new (MX_TYPE_ADJUSTMENT,
                        "value", value,
@@ -398,6 +435,14 @@ mx_adjustment_new (gdouble value,
                        NULL);
 }
 
+/**
+ * mx_adjustment_get_value:
+ * @adjustment: An #MxAdjustment
+ *
+ * Get the current value of the #MxAdjustment:value property
+ *
+ * Returns: the current value of the "value" property
+ */
 gdouble
 mx_adjustment_get_value (MxAdjustment *adjustment)
 {
@@ -407,33 +452,17 @@ mx_adjustment_get_value (MxAdjustment *adjustment)
 
   priv = adjustment->priv;
 
-  if (priv->interpolation)
-    {
-      return MAX (priv->lower,
-                  MIN (priv->upper - priv->page_size,
-                       priv->new_position));
-    }
-  else
-    return priv->value;
+  return priv->value;
 }
 
 static gboolean
 mx_adjustment_value_notify_cb (MxAdjustment *adjustment)
 {
-  ClutterTimeline *interpolation;
   MxAdjustmentPrivate *priv = adjustment->priv;
 
   priv->value_source = 0;
 
-  /* Temporarily set the interpolation timeline to NULL so that
-   * if mx_adjustment_get_value gets called in this callback,
-   * it will get the interpolated value and not just the final
-   * value.
-   */
-  interpolation = priv->interpolation;
-  priv->interpolation = NULL;
   g_object_notify (G_OBJECT (adjustment), "value");
-  priv->interpolation = interpolation;
 
   return FALSE;
 }
@@ -504,6 +533,14 @@ mx_adjustment_emit_changed_cb (MxAdjustment *adjustment)
   return FALSE;
 }
 
+/**
+ * mx_adjustment_set_value:
+ * @adjustment: An #MxAdjustment
+ * @value: A #gdouble
+ *
+ * Set the value of the #MxAdjustment:value property.
+ *
+ */
 void
 mx_adjustment_set_value (MxAdjustment *adjustment,
                          gdouble       value)
@@ -514,8 +551,6 @@ mx_adjustment_set_value (MxAdjustment *adjustment,
 
   priv = adjustment->priv;
 
-  stop_interpolation (adjustment);
-
   /* Defer clamp until after construction. */
   if (!priv->is_constructing)
     {
@@ -527,18 +562,15 @@ mx_adjustment_set_value (MxAdjustment *adjustment,
 
   if (priv->value != value)
     {
+      stop_interpolation (adjustment);
+
       priv->value = value;
 
-      if (!priv->value_source)
-        priv->value_source =
-          g_idle_add_full (CLUTTER_PRIORITY_REDRAW,
-                           (GSourceFunc)mx_adjustment_value_notify_cb,
-                           adjustment,
-                           NULL);
+      g_object_notify (G_OBJECT (adjustment), "value");
     }
 }
 
-void
+static void
 mx_adjustment_clamp_page (MxAdjustment *adjustment,
                           gdouble       lower,
                           gdouble       upper)
@@ -550,8 +582,6 @@ mx_adjustment_clamp_page (MxAdjustment *adjustment,
 
   priv = adjustment->priv;
 
-  stop_interpolation (adjustment);
-
   lower = CLAMP (lower, priv->lower, priv->upper - priv->page_size);
   upper = CLAMP (upper, priv->lower + priv->page_size, priv->upper);
 
@@ -591,8 +621,8 @@ mx_adjustment_emit_changed (MxAdjustment *adjustment)
 }
 
 static gboolean
-mx_adjustment_set_lower (MxAdjustment *adjustment,
-                         gdouble       lower)
+_mx_adjustment_set_lower (MxAdjustment *adjustment,
+                          gdouble       lower)
 {
   MxAdjustmentPrivate *priv = adjustment->priv;
 
@@ -619,9 +649,39 @@ mx_adjustment_set_lower (MxAdjustment *adjustment,
   return FALSE;
 }
 
+/**
+ * mx_adjustment_set_lower:
+ * @adjustment: A #MxAdjustment
+ * @lower: A #gdouble
+ *
+ * Set the value of the #MxAdjustment:lower property.
+ *
+ */
+void
+mx_adjustment_set_lower (MxAdjustment *adjustment,
+                         gdouble       lower)
+{
+  _mx_adjustment_set_lower (adjustment, lower);
+}
+
+/**
+ * mx_adjustment_get_lower:
+ * @adjustment: A #MxAdjustment
+ *
+ * Get the value of the #MxAdjustment:lower property.
+ *
+ * Returns: the current value of the "lower" property.
+ */
+gdouble
+mx_adjustment_get_lower (MxAdjustment *adjustment)
+{
+  g_return_val_if_fail (MX_IS_ADJUSTMENT (adjustment), 0.0);
+  return adjustment->priv->lower;
+}
+
 static gboolean
-mx_adjustment_set_upper (MxAdjustment *adjustment,
-                         gdouble       upper)
+_mx_adjustment_set_upper (MxAdjustment *adjustment,
+                          gdouble       upper)
 {
   MxAdjustmentPrivate *priv = adjustment->priv;
 
@@ -648,9 +708,39 @@ mx_adjustment_set_upper (MxAdjustment *adjustment,
   return FALSE;
 }
 
+/**
+ * mx_adjustment_set_upper:
+ * @adjustment: A #MxAdjustment
+ * @upper: A #gdouble
+ *
+ * Set the value of the #MxAdjustment:upper property.
+ *
+ */
+void
+mx_adjustment_set_upper (MxAdjustment *adjustment,
+                         gdouble       upper)
+{
+  _mx_adjustment_set_upper (adjustment, upper);
+}
+
+/**
+ * mx_adjustment_get_upper:
+ * @adjustment: A #MxAdjustment
+ *
+ * Get the value of the #MxAdjustment:upper property.
+ *
+ * Returns: the current value of the "upper" property.
+ */
+gdouble
+mx_adjustment_get_upper (MxAdjustment *adjustment)
+{
+  g_return_val_if_fail (MX_IS_ADJUSTMENT (adjustment), 0.0);
+  return adjustment->priv->upper;
+}
+
 static gboolean
-mx_adjustment_set_step_increment (MxAdjustment *adjustment,
-                                  gdouble       step)
+_mx_adjustment_set_step_increment (MxAdjustment *adjustment,
+                                   gdouble       step)
 {
   MxAdjustmentPrivate *priv = adjustment->priv;
 
@@ -673,9 +763,39 @@ mx_adjustment_set_step_increment (MxAdjustment *adjustment,
   return FALSE;
 }
 
+/**
+ * mx_adjustment_set_step_increment:
+ * @adjustment: A #MxAdjustment
+ * @increment: A #gdouble
+ *
+ * Set the value of the #MxAdjustment:step-increment property.
+ *
+ */
+void
+mx_adjustment_set_step_increment (MxAdjustment *adjustment,
+                                  gdouble       increment)
+{
+  _mx_adjustment_set_step_increment (adjustment, increment);
+}
+
+/**
+ * mx_adjustment_get_step_increment:
+ * @adjustment: A #MxAdjustment
+ *
+ * Get the value of the MxAdjustment:step-increment property.
+ *
+ * Returns: the current value of the "step-increment" property.
+ */
+gdouble
+mx_adjustment_get_step_increment (MxAdjustment *adjustment)
+{
+  g_return_val_if_fail (MX_IS_ADJUSTMENT (adjustment), 0.0);
+  return adjustment->priv->step_increment;
+}
+
 static gboolean
-mx_adjustment_set_page_increment (MxAdjustment *adjustment,
-                                  gdouble       page)
+_mx_adjustment_set_page_increment (MxAdjustment *adjustment,
+                                   gdouble       page)
 {
   MxAdjustmentPrivate *priv = adjustment->priv;
 
@@ -698,9 +818,39 @@ mx_adjustment_set_page_increment (MxAdjustment *adjustment,
   return FALSE;
 }
 
+/**
+ * mx_adjustment_set_page_increment:
+ * @adjustment: A #MxAdjustment
+ * @increment: A #gdouble
+ *
+ * Set the value of the #MxAdjustment:page-increment property.
+ *
+ */
+void
+mx_adjustment_set_page_increment (MxAdjustment *adjustment,
+                                  gdouble       increment)
+{
+  _mx_adjustment_set_page_increment (adjustment, increment);
+}
+
+/**
+ * mx_adjustment_get_page_increment:
+ * @adjustment: A #MxAdjustment
+ *
+ * Get the value of the MxAdjustment:page-increment property.
+ *
+ * Returns: the current value of the "page-increment" property.
+ */
+gdouble
+mx_adjustment_get_page_increment (MxAdjustment *adjustment)
+{
+  g_return_val_if_fail (MX_IS_ADJUSTMENT (adjustment), 0.0);
+  return adjustment->priv->page_increment;
+}
+
 static gboolean
-mx_adjustment_set_page_size (MxAdjustment *adjustment,
-                             gdouble       size)
+_mx_adjustment_set_page_size (MxAdjustment *adjustment,
+                              gdouble       size)
 {
   MxAdjustmentPrivate *priv = adjustment->priv;
 
@@ -727,6 +877,49 @@ mx_adjustment_set_page_size (MxAdjustment *adjustment,
   return FALSE;
 }
 
+/**
+ * mx_adjustment_set_page_size:
+ * @adjustment: A #MxAdjustment
+ * @page_size: A #gdouble
+ *
+ * Set the #MxAdjustment:page-size property.
+ *
+ */
+void
+mx_adjustment_set_page_size (MxAdjustment *adjustment,
+                             gdouble       page_size)
+{
+  _mx_adjustment_set_page_size (adjustment, page_size);
+}
+
+/**
+ * mx_adjustment_get_page_size:
+ * @adjustment: A #MxAdjustment
+ *
+ * Get the value of the #MxAdjustment:page-size property.
+ *
+ * Returns: the current value of the "page-size" property.
+ */
+gdouble
+mx_adjustment_get_page_size (MxAdjustment *adjustment)
+{
+  g_return_val_if_fail (MX_IS_ADJUSTMENT (adjustment), 0.0);
+  return adjustment->priv->page_size;
+}
+
+/**
+ * mx_adjustment_set_values:
+ * @adjustment: A #MxAdjustment
+ * @value: A #gdouble
+ * @lower: A #gdouble
+ * @upper: A #gdouble
+ * @step_increment: A #gdouble
+ * @page_increment: A #gdouble
+ * @page_size: A #gdouble
+ *
+ * Set the various properties of MxAdjustment.
+ *
+ */
 void
 mx_adjustment_set_values (MxAdjustment *adjustment,
                           gdouble       value,
@@ -746,17 +939,17 @@ mx_adjustment_set_values (MxAdjustment *adjustment,
 
   priv = adjustment->priv;
 
-  stop_interpolation (adjustment);
-
   emit_changed = FALSE;
 
   g_object_freeze_notify (G_OBJECT (adjustment));
 
-  emit_changed |= mx_adjustment_set_lower (adjustment, lower);
-  emit_changed |= mx_adjustment_set_upper (adjustment, upper);
-  emit_changed |= mx_adjustment_set_step_increment (adjustment, step_increment);
-  emit_changed |= mx_adjustment_set_page_increment (adjustment, page_increment);
-  emit_changed |= mx_adjustment_set_page_size (adjustment, page_size);
+  emit_changed |= _mx_adjustment_set_lower (adjustment, lower);
+  emit_changed |= _mx_adjustment_set_upper (adjustment, upper);
+  emit_changed |= _mx_adjustment_set_step_increment (adjustment,
+                                                     step_increment);
+  emit_changed |= _mx_adjustment_set_page_increment (adjustment,
+                                                     page_increment);
+  emit_changed |= _mx_adjustment_set_page_size (adjustment, page_size);
 
   if (value != priv->value)
     {
@@ -770,6 +963,19 @@ mx_adjustment_set_values (MxAdjustment *adjustment,
   g_object_thaw_notify (G_OBJECT (adjustment));
 }
 
+/**
+ * mx_adjustment_get_values:
+ * @adjustment: A #MxAdjustment
+ * @value: A #gdouble
+ * @lower: A #gdouble
+ * @upper: A #gdouble
+ * @step_increment: A #gdouble
+ * @page_increment: A #gdouble
+ * @page_size: A #gdouble
+ *
+ * Get the various properties of MxAdjustment.
+ *
+ */
 void
 mx_adjustment_get_values (MxAdjustment *adjustment,
                           gdouble      *value,
@@ -826,7 +1032,7 @@ interpolation_new_frame_cb (ClutterTimeline *timeline,
     mx_adjustment_set_value (adjustment,
                              priv->old_position +
                              (priv->new_position - priv->old_position) *
-                             clutter_timeline_get_progress (timeline));
+                             clutter_alpha_get_alpha (priv->interpolate_alpha));
 
   priv->interpolation = timeline;
 }
@@ -863,10 +1069,21 @@ interpolation_completed_cb (ClutterTimeline *timeline,
    }
  */
 
+/**
+ * mx_adjustment_interpolate:
+ * @adjustment: A #MxAdjustment
+ * @value: A #gdouble
+ * @duration: duration in milliseconds
+ * @mode: A #ClutterAnimationMode
+ *
+ * Interpolate #MxAdjustment:value to the new value specified by @value, using
+ * the mode and duration given.
+ */
 void
 mx_adjustment_interpolate (MxAdjustment *adjustment,
                            gdouble       value,
-                           guint         duration)
+                           guint         duration,
+                           gulong        mode)
 {
   MxAdjustmentPrivate *priv = adjustment->priv;
 
@@ -887,6 +1104,12 @@ mx_adjustment_interpolate (MxAdjustment *adjustment,
     priv->bounce_alpha = clutter_alpha_new_full (priv->interpolation,
                                                  CLUTTER_LINEAR);
 
+  if (priv->interpolate_alpha)
+    g_object_unref (priv->interpolate_alpha);
+
+  priv->interpolate_alpha = clutter_alpha_new_full (priv->interpolation,
+                                                    mode);
+
   g_signal_connect (priv->interpolation,
                     "new-frame",
                     G_CALLBACK (interpolation_new_frame_cb),
@@ -899,12 +1122,58 @@ mx_adjustment_interpolate (MxAdjustment *adjustment,
   clutter_timeline_start (priv->interpolation);
 }
 
+/**
+ * mx_adjustment_interpolate_relative:
+ * @adjustment: A #MxAdjustment
+ * @offset: A #gdouble
+ * @duration: duration in milliseconds
+ * @mode: A #ClutterAnimationMode
+ *
+ * Interpolate the value of #MxAdjustment:value to a new value calculated from
+ * @offset.
+ *
+ */
+void
+mx_adjustment_interpolate_relative (MxAdjustment *adjustment,
+                                    gdouble       offset,
+                                    guint         duration,
+                                    gulong        mode)
+{
+  MxAdjustmentPrivate *priv = adjustment->priv;
+
+  if (priv->interpolation)
+    offset += priv->new_position;
+  else
+    offset += priv->value;
+
+  mx_adjustment_interpolate (adjustment,
+                             offset,
+                             duration,
+                             mode);
+}
+
+/**
+ * mx_adjustment_get_elastic:
+ * @adjustment: A #MxAdjustment
+ *
+ * Get the value of the #MxAdjustment:elastic property.
+ *
+ * Returns: the current value of the "elastic" property.
+ */
 gboolean
 mx_adjustment_get_elastic (MxAdjustment *adjustment)
 {
   return adjustment->priv->elastic;
 }
 
+/**
+ * mx_adjustment_set_elastic:
+ * @adjustment: A #MxAdjustment
+ * @elastic: A #gboolean
+ *
+ * Set the value of the #MxAdjustment:elastic property.
+ *
+ */
 void
 mx_adjustment_set_elastic (MxAdjustment *adjustment,
                            gboolean      elastic)
@@ -912,29 +1181,3 @@ mx_adjustment_set_elastic (MxAdjustment *adjustment,
   adjustment->priv->elastic = elastic;
 }
 
-gboolean
-mx_adjustment_clamp (MxAdjustment *adjustment,
-                     gboolean      interpolate,
-                     guint         duration)
-{
-  MxAdjustmentPrivate *priv = adjustment->priv;
-  gdouble dest = priv->value;
-
-  if (priv->value < priv->lower)
-    dest = priv->lower;
-
-  if (priv->value > priv->upper - priv->page_size)
-    dest = priv->upper - priv->page_size;
-
-  if (dest != priv->value)
-    {
-      if (interpolate)
-        mx_adjustment_interpolate (adjustment, dest, duration);
-      else
-        mx_adjustment_set_value (adjustment, dest);
-
-      return TRUE;
-    }
-
-  return FALSE;
-}
