@@ -181,8 +181,8 @@ mx_application_register_dbus (MxApplication *application,
 
       if (first_run)
         first_run = FALSE;
-/*      else
-        dbus_g_connection_unregister_g_object (bus, G_OBJECT (application));*/
+      else
+        dbus_g_connection_unregister_g_object (bus, G_OBJECT (application));
     }
   else
     {
