@@ -1,7 +1,8 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-box-layout.h: box layout actor
  *
- * Copyright 2009 Intel Corporation.
+ * Copyright 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -34,36 +35,43 @@
  * <figure id="mx-box-layout-horizontal">
  *   <title>Box layout with horizontal flow</title>
  *   <para>The image shows an #MxBoxLayout with the
- *   #MxBoxLayout:vertical property set to
- *   #FALSE.</para>
+ *   #MxBoxLayout:orientation property set to
+ *   #MX_ORIENTATION_HORIZONTAL.</para>
  *   <graphic fileref="MxBoxLayout-horizontal.png" format="PNG"/>
  * </figure>
  *
  * <figure id="mx-box-layout-vertical">
  *   <title>Box layout with vertical flow</title>
  *   <para>The image shows an #MxBoxLayout with the
- *   #MxBoxLayout:vertical property set to
- *   #TRUE.</para>
+ *   #MxBoxLayout:orientation property set to
+ *   #MX_ORIENTATION_VERTICAL.</para>
  *   <graphic fileref="MxBoxLayout-vertical.png" format="PNG"/>
  * </figure>
  */
 
+#include <stdarg.h>
+#include <string.h>
+#include <gobject/gvaluecollector.h>
+
 #include "mx-box-layout.h"
 
 #include "mx-private.h"
 #include "mx-scrollable.h"
 #include "mx-box-layout-child.h"
-
+#include "mx-focusable.h"
 
 
 static void mx_box_container_iface_init (ClutterContainerIface *iface);
-static void mx_box_scrollable_interface_init (MxScrollableInterface *iface);
+static void mx_box_scrollable_interface_init (MxScrollableIface *iface);
+static void mx_box_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxBoxLayout, mx_box_layout, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
                                                 mx_box_container_iface_init)
                          G_IMPLEMENT_INTERFACE (MX_TYPE_SCROLLABLE,
-                                                mx_box_scrollable_interface_init));
+                                                mx_box_scrollable_interface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_box_focusable_iface_init));
 
 #define BOX_LAYOUT_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MX_TYPE_BOX_LAYOUT, MxBoxLayoutPrivate))
@@ -71,8 +79,7 @@ G_DEFINE_TYPE_WITH_CODE (MxBoxLayout, mx_box_layout, MX_TYPE_WIDGET,
 enum {
   PROP_0,
 
-  PROP_VERTICAL,
-  PROP_PACK_START,
+  PROP_ORIENTATION,
   PROP_SPACING,
 
   PROP_HADJUST,
@@ -87,9 +94,6 @@ struct _MxBoxLayoutPrivate
 
   guint         spacing;
 
-  guint         is_vertical : 1;
-  guint         is_pack_start : 1;
-
   MxAdjustment *hadjustment;
   MxAdjustment *vadjustment;
 
@@ -98,6 +102,10 @@ struct _MxBoxLayoutPrivate
   ClutterAlpha    *alpha;
   guint            is_animating : 1;
   guint            enable_animations : 1;
+
+  MxOrientation orientation;
+
+  MxFocusable *last_focus;
 };
 
 void _mx_box_layout_finish_animation (MxBoxLayout *box);
@@ -228,17 +236,14 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
           /* create an initial adjustment. this is filled with correct values
            * as soon as allocate() is called */
 
-          adjustment = mx_adjustment_new (0.0,
-                                          0.0,
-                                          1.0,
-                                          1.0,
-                                          1.0,
-                                          1.0);
+          adjustment = mx_adjustment_new ();
 
           scrollable_set_adjustments (scrollable,
                                       adjustment,
                                       priv->vadjustment);
 
+          g_object_unref (adjustment);
+
           *hadjustment = adjustment;
         }
     }
@@ -254,17 +259,14 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
           /* create an initial adjustment. this is filled with correct values
            * as soon as allocate() is called */
 
-          adjustment = mx_adjustment_new (0.0,
-                                          0.0,
-                                          1.0,
-                                          1.0,
-                                          1.0,
-                                          1.0);
+          adjustment = mx_adjustment_new ();
 
           scrollable_set_adjustments (scrollable,
                                       priv->hadjustment,
                                       adjustment);
 
+          g_object_unref (adjustment);
+
           *vadjustment = adjustment;
         }
     }
@@ -273,7 +275,7 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
 
 
 static void
-mx_box_scrollable_interface_init (MxScrollableInterface *iface)
+mx_box_scrollable_interface_init (MxScrollableIface *iface)
 {
   iface->set_adjustments = scrollable_set_adjustments;
   iface->get_adjustments = scrollable_get_adjustments;
@@ -337,10 +339,16 @@ mx_box_container_remove_actor (ClutterContainer *container,
 
   priv = MX_BOX_LAYOUT (container)->priv;
 
+  if ((ClutterActor *)priv->last_focus == actor)
+    priv->last_focus = NULL;
+
   priv->children = g_list_delete_link (priv->children, item);
   clutter_actor_unparent (actor);
 
-  _mx_box_layout_start_animation (MX_BOX_LAYOUT (container));
+  if (priv->enable_animations)
+    _mx_box_layout_start_animation (MX_BOX_LAYOUT (container));
+  else
+    clutter_actor_queue_relayout ((ClutterActor *) container);
 
   g_signal_emit_by_name (container, "actor-removed", actor);
 
@@ -442,6 +450,188 @@ mx_box_container_iface_init (ClutterContainerIface *iface)
   iface->child_meta_type = MX_TYPE_BOX_LAYOUT_CHILD;
 }
 
+/*
+ * focusable implementation
+ */
+
+static void
+update_adjustments (MxBoxLayout *self,
+                    MxFocusable *focusable)
+{
+  MxBoxLayoutPrivate *priv = self->priv;
+  gdouble value, new_value, page_size;
+  ClutterActorBox box = { 0, };
+  clutter_actor_get_allocation_box (CLUTTER_ACTOR (focusable), &box);
+
+  if (priv->vadjustment)
+    {
+      mx_adjustment_get_values (priv->vadjustment,
+                                &value, NULL, NULL, NULL, NULL,
+                                &page_size);
+      if (box.y1 < value)
+        new_value = box.y1;
+      else if (box.y2 > value + page_size)
+        new_value = box.y2 - page_size;
+      else
+        new_value = value;
+      mx_adjustment_interpolate (priv->vadjustment,
+                                 new_value,
+                                 250, CLUTTER_EASE_OUT_CUBIC);
+    }
+
+  if (priv->hadjustment)
+    {
+      mx_adjustment_get_values (priv->hadjustment,
+                                &value, NULL, NULL, NULL, NULL,
+                                &page_size);
+      if (box.x1 < value)
+        new_value = box.x1;
+      else if (box.x2 > value + page_size)
+        new_value = box.x2 - page_size;
+      else
+        new_value = value;
+      mx_adjustment_interpolate (priv->hadjustment,
+                                 new_value,
+                                 250, CLUTTER_EASE_OUT_CUBIC);
+    }
+}
+
+static MxFocusable*
+mx_box_layout_move_focus (MxFocusable      *focusable,
+                          MxFocusDirection  direction,
+                          MxFocusable      *from)
+{
+  MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (focusable)->priv;
+  GList *l, *childlink;
+
+  /* find the current focus */
+  childlink = g_list_find (priv->children, from);
+
+  if (!childlink)
+    return NULL;
+
+  /* convert left/right and up/down into next/previous */
+  if (priv->orientation == MX_ORIENTATION_HORIZONTAL)
+    {
+      if (direction == MX_FOCUS_DIRECTION_LEFT)
+        direction = MX_FOCUS_DIRECTION_PREVIOUS;
+      else if (direction == MX_FOCUS_DIRECTION_RIGHT)
+        direction = MX_FOCUS_DIRECTION_NEXT;
+    }
+  else
+    {
+      if (direction == MX_FOCUS_DIRECTION_UP)
+        direction = MX_FOCUS_DIRECTION_PREVIOUS;
+      else if (direction == MX_FOCUS_DIRECTION_DOWN)
+        direction = MX_FOCUS_DIRECTION_NEXT;
+    }
+
+  /* find the next widget to focus */
+  if (direction == MX_FOCUS_DIRECTION_NEXT)
+    {
+      for (l = childlink->next; l; l = g_list_next (l))
+        {
+          if (MX_IS_FOCUSABLE (l->data))
+            {
+              MxFocusable *focused, *child;
+
+              child = MX_FOCUSABLE (l->data);
+              focused = mx_focusable_accept_focus (child, MX_FOCUS_HINT_FIRST);
+
+              if (focused)
+                {
+                  priv->last_focus = child;
+                  update_adjustments (MX_BOX_LAYOUT (focusable), focused);
+                  return focused;
+                }
+            }
+        }
+
+      /* no next widgets to focus */
+      return NULL;
+    }
+  else if (direction == MX_FOCUS_DIRECTION_PREVIOUS)
+    {
+      for (l = g_list_previous (childlink); l; l = g_list_previous (l))
+        {
+          if (MX_IS_FOCUSABLE (l->data))
+            {
+              MxFocusable *focused, *child;
+
+              child = MX_FOCUSABLE (l->data);
+              focused = mx_focusable_accept_focus (child, MX_FOCUS_HINT_LAST);
+
+              if (focused)
+                {
+                  priv->last_focus = child;
+                  update_adjustments (MX_BOX_LAYOUT (focusable), focused);
+                  return focused;
+                }
+            }
+        }
+    }
+
+  return NULL;
+}
+
+static MxFocusable*
+mx_box_layout_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (focusable)->priv;
+  MxFocusable *return_focusable;
+  GList* list, *l;
+
+  return_focusable = NULL;
+
+  /* find the first/last focusable widget */
+  switch (hint)
+    {
+    case MX_FOCUS_HINT_LAST:
+      list = g_list_reverse (g_list_copy (priv->children));
+      break;
+
+    case MX_FOCUS_HINT_PRIOR:
+      if (priv->last_focus)
+        {
+          list = g_list_copy (g_list_find (priv->children, priv->last_focus));
+          if (list)
+            break;
+        }
+      /* This intentionally runs into the next case */
+
+    default:
+    case MX_FOCUS_HINT_FIRST:
+      list = g_list_copy (priv->children);
+      break;
+    }
+
+  for (l = list; l; l = g_list_next (l))
+    {
+      if (MX_IS_FOCUSABLE (l->data))
+        {
+          return_focusable = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                        hint);
+
+          if (return_focusable)
+            {
+              update_adjustments (MX_BOX_LAYOUT (focusable), return_focusable);
+              break;
+            }
+        }
+    }
+
+  g_list_free (list);
+
+  return return_focusable;
+}
+
+static void
+mx_box_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->move_focus = mx_box_layout_move_focus;
+  iface->accept_focus = mx_box_layout_accept_focus;
+}
+
 
 static void
 mx_box_layout_get_property (GObject    *object,
@@ -454,12 +644,8 @@ mx_box_layout_get_property (GObject    *object,
 
   switch (property_id)
     {
-    case PROP_VERTICAL:
-      g_value_set_boolean (value, priv->is_vertical);
-      break;
-
-    case PROP_PACK_START:
-      g_value_set_boolean (value, priv->is_pack_start);
+    case PROP_ORIENTATION:
+      g_value_set_enum (value, priv->orientation);
       break;
 
     case PROP_SPACING:
@@ -495,12 +681,8 @@ mx_box_layout_set_property (GObject      *object,
 
   switch (property_id)
     {
-    case PROP_VERTICAL:
-      mx_box_layout_set_vertical (box, g_value_get_boolean (value));
-      break;
-
-    case PROP_PACK_START:
-      mx_box_layout_set_pack_start (box, g_value_get_boolean (value));
+    case PROP_ORIENTATION:
+      mx_box_layout_set_orientation (box, g_value_get_enum (value));
       break;
 
     case PROP_SPACING:
@@ -533,6 +715,9 @@ mx_box_layout_dispose (GObject *object)
 {
   MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (object)->priv;
 
+  /* cleanup any running animation */
+  _mx_box_layout_finish_animation (MX_BOX_LAYOUT (object));
+
   /* destroy the children
    * clutter_actor_destroy() will call clutter_container_remove() which will
    * remove the children from the internal list
@@ -591,18 +776,24 @@ mx_box_layout_get_preferred_width (ClutterActor *actor,
   for (l = priv->children; l; l = g_list_next (l))
     {
       gfloat child_min = 0, child_nat = 0;
+      gfloat child_for_height;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE ((ClutterActor*) l->data))
         continue;
 
       n_children++;
 
+      if (priv->orientation == MX_ORIENTATION_HORIZONTAL)
+        child_for_height = for_height;
+      else
+        child_for_height = -1;
+
       clutter_actor_get_preferred_width ((ClutterActor*) l->data,
-                                         (!priv->is_vertical) ? for_height : -1,
+                                         child_for_height,
                                          &child_min,
                                          &child_nat);
 
-      if (priv->is_vertical)
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
         {
           if (min_width_p)
             *min_width_p = MAX (child_min, *min_width_p);
@@ -622,7 +813,7 @@ mx_box_layout_get_preferred_width (ClutterActor *actor,
     }
 
 
-  if (!priv->is_vertical && n_children > 1)
+  if (priv->orientation == MX_ORIENTATION_HORIZONTAL && n_children > 1)
     {
       if (min_width_p)
         *min_width_p += priv->spacing * (n_children - 1);
@@ -662,18 +853,24 @@ mx_box_layout_get_preferred_height (ClutterActor *actor,
   for (l = priv->children; l; l = g_list_next (l))
     {
       gfloat child_min = 0, child_nat = 0;
+      gfloat child_for_width;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE ((ClutterActor*) l->data))
         continue;
 
       n_children++;
 
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
+        child_for_width = for_width;
+      else
+        child_for_width = -1;
+
       clutter_actor_get_preferred_height ((ClutterActor*) l->data,
-                                          (priv->is_vertical) ? for_width : -1,
+                                          child_for_width,
                                           &child_min,
                                           &child_nat);
 
-      if (!priv->is_vertical)
+      if (priv->orientation == MX_ORIENTATION_HORIZONTAL)
         {
           if (min_height_p)
             *min_height_p = MAX (child_min, *min_height_p);
@@ -691,7 +888,7 @@ mx_box_layout_get_preferred_height (ClutterActor *actor,
         }
     }
 
-  if (priv->is_vertical && n_children > 1)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL && n_children > 1)
     {
       if (min_height_p)
         *min_height_p += priv->spacing * (n_children - 1);
@@ -715,9 +912,11 @@ mx_box_layout_allocate (ClutterActor          *actor,
   MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (actor)->priv;
   gfloat avail_width, avail_height, pref_width, pref_height;
   MxPadding padding = { 0, };
+  gboolean allocate_pref;
+  gfloat extra_space = 0;
   gfloat position = 0;
   GList *l;
-  gint n_expand_children, extra_space;
+  gint n_expand_children, n_children;
 
   CLUTTER_ACTOR_CLASS (mx_box_layout_parent_class)->allocate (actor, box,
                                                               flags);
@@ -725,108 +924,177 @@ mx_box_layout_allocate (ClutterActor          *actor,
   if (priv->children == NULL)
     return;
 
+  /* count the number of children with expand set to TRUE and the
+   * amount of visible children.
+   */
+  n_children = n_expand_children = 0;
+  for (l = priv->children; l; l = l->next)
+    {
+      ClutterActor *child;
+
+      child = (ClutterActor*) l->data;
+
+      if (CLUTTER_ACTOR_IS_VISIBLE (child))
+        {
+          gboolean expand;
+
+          clutter_container_child_get ((ClutterContainer *) actor,
+                                       child,
+                                       "expand", &expand,
+                                       NULL);
+          n_children++;
+
+          if (expand)
+            n_expand_children++;
+
+        }
+    }
+
+  /* We have no visible children, so bail out */
+  if (n_children == 0)
+    return;
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
-  avail_width  = box->x2 - box->x1
-                 - padding.left
-                 - padding.right;
-  avail_height = box->y2 - box->y1
-                 - padding.top
-                 - padding.bottom;
 
-  if (priv->is_vertical)
+  /* do not take off padding just yet, as we are comparing this to the values
+   * from get_preferred_height/width which will include padding */
+  avail_width  = box->x2 - box->x1;
+  avail_height = box->y2 - box->y1;
+
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
-      mx_box_layout_get_preferred_height (actor, avail_width, NULL,
-                                          &pref_height);
+      gfloat min_height;
+      mx_box_layout_get_preferred_height (actor, avail_width,
+                                          &min_height, &pref_height);
       pref_width = avail_width;
+
+      if (!priv->vadjustment && (pref_height > box->y2 - box->y1))
+        {
+          /* allocated less than the preferred height and not scrolling */
+          allocate_pref = FALSE;
+          extra_space = avail_height - min_height;
+        }
+      else
+        allocate_pref = TRUE;
     }
   else
     {
-      mx_box_layout_get_preferred_width (actor, avail_height, NULL,
-                                         &pref_width);
+      gfloat min_width;
+      mx_box_layout_get_preferred_width (actor, avail_height,
+                                         &min_width, &pref_width);
       pref_height = avail_height;
+
+      if (!priv->hadjustment && (pref_width > box->x2 - box->x1))
+        {
+          /* allocated less than the preferred width and not scrolling */
+          allocate_pref = FALSE;
+          extra_space = avail_width - min_width;
+        }
+      else
+        allocate_pref = TRUE;
     }
 
+  /* remove the padding values from the available and preferred sizes so we
+   * can use them for allocating the children */
+  avail_width -= padding.left + padding.right;
+  avail_height -= padding.top + padding.bottom;
+
+  pref_width -= padding.left + padding.right;
+  pref_height -= padding.top + padding.bottom;
+
   /* update adjustments for scrolling */
   if (priv->vadjustment)
     {
-      gdouble prev_value;
+      gdouble step_inc, page_inc;
+
+      /* Base the adjustment stepping on the size of the first child.
+       * In the case where all your children are the same size, this
+       * will probably provide the desired behaviour.
+       */
+      if (priv->children && priv->orientation == MX_ORIENTATION_VERTICAL)
+        {
+          gfloat child_height;
+          ClutterActor *first_child = (ClutterActor *)priv->children->data;
+          clutter_actor_get_preferred_height (first_child,
+                                              avail_width,
+                                              NULL,
+                                              &child_height);
+          step_inc = child_height;
+          page_inc = ((gint)(avail_height / step_inc)) * step_inc;
+        }
+      else
+        {
+          step_inc = avail_height / 6;
+          page_inc = avail_height;
+        }
 
       g_object_set (G_OBJECT (priv->vadjustment),
                     "lower", 0.0,
                     "upper", pref_height,
                     "page-size", avail_height,
-                    "step-increment", avail_height / 6,
-                    "page-increment", avail_height,
+                    "step-increment", step_inc,
+                    "page-increment", page_inc,
                     NULL);
-
-      prev_value = mx_adjustment_get_value (priv->vadjustment);
-      mx_adjustment_set_value (priv->vadjustment, prev_value);
     }
 
   if (priv->hadjustment)
     {
-      gdouble prev_value;
+      gdouble step_inc, page_inc;
+
+      if (priv->children && priv->orientation == MX_ORIENTATION_HORIZONTAL)
+        {
+          gfloat child_width;
+          ClutterActor *first_child = (ClutterActor *)priv->children->data;
+          clutter_actor_get_preferred_width (first_child,
+                                             avail_height,
+                                             &child_width,
+                                             NULL);
+          step_inc = child_width;
+          page_inc = ((gint)(avail_width / step_inc)) * step_inc;
+        }
+      else
+        {
+          step_inc = avail_width / 6;
+          page_inc = avail_width;
+        }
 
       g_object_set (G_OBJECT (priv->hadjustment),
                     "lower", 0.0,
                     "upper", pref_width,
                     "page-size", avail_width,
-                    "step-increment", avail_width / 6,
-                    "page-increment", avail_width,
+                    "step-increment", step_inc,
+                    "page-increment", page_inc,
                     NULL);
-
-      prev_value = mx_adjustment_get_value (priv->hadjustment);
-      mx_adjustment_set_value (priv->hadjustment, prev_value);
     }
 
-  /* count the number of children with expand set to TRUE */
-  n_expand_children = 0;
-  for (l = priv->children; l; l = l->next)
+  /* We're allocating our preferred size or higher, so calculate
+   * the extra space to give to expanded children.
+   */
+  if (allocate_pref)
     {
-      gboolean expand;
-      clutter_container_child_get ((ClutterContainer *) actor,
-                                   (ClutterActor*) l->data,
-                                   "expand", &expand,
-                                   NULL);
-      if (expand)
-        n_expand_children++;
+      if (n_expand_children > 0)
+        {
+          if (priv->orientation == MX_ORIENTATION_VERTICAL)
+            extra_space = (avail_height - pref_height) / n_expand_children;
+          else
+            extra_space = (avail_width - pref_width) / n_expand_children;
+
+          /* don't shrink anything */
+          if (extra_space < 0)
+            extra_space = 0;
+        }
     }
 
-  if (n_expand_children == 0)
-    {
-      extra_space = 0;
-      n_expand_children = 1;
-    }
-  else
-    {
-      if (priv->is_vertical)
-        extra_space = (avail_height - pref_height) / n_expand_children;
-      else
-        extra_space = (avail_width - pref_width) / n_expand_children;
-
-      /* don't shrink anything */
-      if (extra_space < 0)
-        extra_space = 0;
-    }
-
-  if (priv->is_vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     position = padding.top;
   else
     position = padding.left;
 
-  if (priv->is_pack_start)
-    l = g_list_last (priv->children);
-  else
-    l = priv->children;
-
-  for (l = (priv->is_pack_start) ? g_list_last (priv->children) : priv->children;
-       l;
-       l = (priv->is_pack_start) ? l->prev : l->next)
+  for (l = priv->children; l; l = l->next)
     {
       ClutterActor *child = (ClutterActor*) l->data;
-      ClutterActorBox child_box;
-      gfloat child_nat;
+      ClutterActorBox child_box, old_child_box;
+      gfloat child_nat, child_min;
       gboolean xfill, yfill, expand;
       MxAlign xalign, yalign;
 
@@ -841,37 +1109,70 @@ mx_box_layout_allocate (ClutterActor          *actor,
                                    "expand", &expand,
                                    NULL);
 
-      if (priv->is_vertical)
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
         {
           clutter_actor_get_preferred_height (child, avail_width,
-                                              NULL, &child_nat);
+                                              &child_min, &child_nat);
 
           child_box.y1 = position;
-          if (expand)
-            child_box.y2 = position + child_nat + extra_space;
+
+          if (allocate_pref)
+            {
+              if (expand)
+                child_box.y2 = position + child_nat + (int)extra_space;
+              else
+                child_box.y2 = position + child_nat;
+            }
           else
-            child_box.y2 = position + child_nat;
+            {
+              child_box.y2 =
+                position + MIN (child_nat,
+                                child_min + (int)(extra_space / n_children));
+
+              n_children --;
+              if (extra_space >= (child_box.y2 - child_box.y1 - child_min))
+                extra_space -= child_box.y2 - child_box.y1 - child_min;
+              else
+                extra_space = 0;
+            }
+
           child_box.x1 = padding.left;
           child_box.x2 = avail_width + padding.left;
         }
       else
         {
-
           clutter_actor_get_preferred_width (child, avail_height,
-                                             NULL, &child_nat);
+                                             &child_min, &child_nat);
 
           child_box.x1 = position;
 
-          if (expand)
-            child_box.x2 = position + child_nat + extra_space;
+          if (allocate_pref)
+            {
+              if (expand)
+                child_box.x2 = position + child_nat + (int)extra_space;
+              else
+                child_box.x2 = position + child_nat;
+            }
           else
-            child_box.x2 = position + child_nat;
+            {
+              child_box.x2 =
+                position + MIN (child_nat,
+                                child_min + (int)(extra_space / n_children));
+
+              n_children --;
+              if (extra_space >= (child_box.x2 - child_box.x1 - child_min))
+                extra_space -= child_box.x2 - child_box.x1 - child_min;
+              else
+                extra_space = 0;
+            }
 
           child_box.y1 = padding.top;
           child_box.y2 = avail_height + padding.top;
         }
 
-      _mx_allocate_fill (child, &child_box, xalign, yalign, xfill, yfill);
+      /* Adjust the box for alignment/fill */
+      old_child_box = child_box;
+      mx_allocate_align_fill (child, &child_box, xalign, yalign, xfill, yfill);
 
       if (priv->is_animating)
         {
@@ -921,11 +1222,10 @@ mx_box_layout_allocate (ClutterActor          *actor,
           clutter_actor_allocate (child, copy, flags);
         }
 
-      if (expand)
-        position += (child_nat + priv->spacing + extra_space);
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
+        position += (old_child_box.y2 - old_child_box.y1) + priv->spacing;
       else
-        position += (child_nat + priv->spacing);
-
+        position += (old_child_box.x2 - old_child_box.x1) + priv->spacing;
     }
 }
 
@@ -1073,20 +1373,13 @@ mx_box_layout_class_init (MxBoxLayoutClass *klass)
   actor_class->paint = mx_box_layout_paint;
   actor_class->pick = mx_box_layout_pick;
 
-  pspec = g_param_spec_boolean ("vertical",
-                                "Vertical",
-                                "Whether the layout should be vertical, rather "
-                                "than horizontal",
-                                FALSE,
-                                MX_PARAM_READWRITE);
-  g_object_class_install_property (object_class, PROP_VERTICAL, pspec);
-
-  pspec = g_param_spec_boolean ("pack-start",
-                                "Pack Start",
-                                "Whether to pack items at the start of the box",
-                                FALSE,
-                                MX_PARAM_READWRITE);
-  g_object_class_install_property (object_class, PROP_PACK_START, pspec);
+  pspec = g_param_spec_enum ("orientation",
+                             "Orientation",
+                             "Orientation of the layout",
+                             MX_TYPE_ORIENTATION,
+                             MX_ORIENTATION_HORIZONTAL,
+                             MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_ORIENTATION, pspec);
 
   pspec = g_param_spec_uint ("spacing",
                              "Spacing",
@@ -1106,11 +1399,11 @@ mx_box_layout_class_init (MxBoxLayoutClass *klass)
   /* MxScrollable properties */
   g_object_class_override_property (object_class,
                                     PROP_HADJUST,
-                                    "hadjustment");
+                                    "horizontal-adjustment");
 
   g_object_class_override_property (object_class,
                                     PROP_VADJUST,
-                                    "vadjustment");
+                                    "vertical-adjustment");
 
 }
 
@@ -1148,82 +1441,43 @@ mx_box_layout_new (void)
 }
 
 /**
- * mx_box_layout_set_vertical:
+ * mx_box_layout_set_orientation:
  * @box: A #MxBoxLayout
- * @vertical: #TRUE if the layout should be vertical
+ * @orientation: orientation value for the layout
  *
- * Set the value of the #MxBoxLayout::vertical property
+ * Set the orientation of the box layout.
  *
  */
 void
-mx_box_layout_set_vertical (MxBoxLayout *box,
-                            gboolean     vertical)
+mx_box_layout_set_orientation (MxBoxLayout *box,
+                               MxOrientation orientation)
 {
   g_return_if_fail (MX_IS_BOX_LAYOUT (box));
 
-  if (box->priv->is_vertical != vertical)
+  if (box->priv->orientation != orientation)
     {
-      box->priv->is_vertical = vertical;
+      box->priv->orientation = orientation;
       _mx_box_layout_start_animation (box);
-      clutter_actor_queue_relayout ((ClutterActor*) box);
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (box));
 
-      g_object_notify (G_OBJECT (box), "vertical");
+      g_object_notify (G_OBJECT (box), "orientation");
     }
 }
 
 /**
- * mx_box_layout_get_vertical:
+ * mx_box_layout_get_orientation:
  * @box: A #MxBoxLayout
  *
- * Get the value of the #MxBoxLayout:vertical property.
+ * Get the value of the #MxBoxLayout:orientation property.
  *
- * Returns: #TRUE if the layout is vertical
+ * Returns: the orientation of the layout
  */
-gboolean
-mx_box_layout_get_vertical (MxBoxLayout *box)
+MxOrientation
+mx_box_layout_get_orientation (MxBoxLayout *box)
 {
   g_return_val_if_fail (MX_IS_BOX_LAYOUT (box), FALSE);
 
-  return box->priv->is_vertical;
-}
-
-/**
- * mx_box_layout_set_pack_start:
- * @box: A #MxBoxLayout
- * @pack_start: #TRUE if the layout should use pack-start
- *
- * Set the value of the #MxBoxLayout:pack-start property.
- *
- */
-void
-mx_box_layout_set_pack_start (MxBoxLayout *box,
-                              gboolean     pack_start)
-{
-  g_return_if_fail (MX_IS_BOX_LAYOUT (box));
-
-  if (box->priv->is_pack_start != pack_start)
-    {
-      box->priv->is_pack_start = pack_start;
-      clutter_actor_queue_relayout ((ClutterActor*) box);
-
-      g_object_notify (G_OBJECT (box), "pack-start");
-    }
-}
-
-/**
- * mx_box_layout_get_pack_start:
- * @box: A #MxBoxLayout
- *
- * Get the value of the #MxBoxLayout:pack-start property.
- *
- * Returns: #TRUE if pack-start is enabled
- */
-gboolean
-mx_box_layout_get_pack_start (MxBoxLayout *box)
-{
-  g_return_val_if_fail (MX_IS_BOX_LAYOUT (box), FALSE);
-
-  return box->priv->is_pack_start;
+  return box->priv->orientation;
 }
 
 /**
@@ -1309,4 +1563,159 @@ mx_box_layout_get_enable_animations (MxBoxLayout *box)
   return box->priv->enable_animations;
 }
 
+static inline void
+mx_box_layout_set_property_valist (MxBoxLayout  *box,
+                                   ClutterActor *actor,
+                                   const gchar  *first_property,
+                                   va_list       var_args)
+{
+  ClutterContainer *container = CLUTTER_CONTAINER (box);
+  ClutterChildMeta *meta;
+  GObjectClass *klass;
+  const gchar *pname;
 
+  meta = clutter_container_get_child_meta (container, actor);
+  g_assert (meta);
+
+  klass = G_OBJECT_GET_CLASS (meta);
+
+  pname = first_property;
+  while (pname)
+    {
+      GValue value = { 0, };
+      GParamSpec *pspec;
+      gchar *error;
+
+      pspec = g_object_class_find_property (klass, pname);
+      if (pspec == NULL)
+        {
+          g_warning ("%s: the layout property '%s' for MxBoxLayout "
+                     "(meta type '%s') does not exist",
+                     G_STRLOC,
+                     pname,
+                     G_OBJECT_TYPE_NAME (meta));
+          break;
+        }
+
+      if (!(pspec->flags & G_PARAM_WRITABLE))
+        {
+          g_warning ("%s: the layout property '%s' for MxBoxLayout "
+                     "(meta type '%s') is not writable",
+                     G_STRLOC,
+                     pspec->name,
+                     G_OBJECT_TYPE_NAME (meta));
+          break;
+        }
+
+      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+      G_VALUE_COLLECT (&value, var_args, 0, &error);
+      if (error)
+        {
+          g_warning ("%s: %s", G_STRLOC, error);
+          g_free (error);
+          break;
+        }
+
+      clutter_container_child_set_property (container,
+                                            actor,
+                                            pspec->name, &value);
+      g_value_unset (&value);
+
+      pname = va_arg (var_args, gchar*);
+    }
+}
+
+static void
+mx_box_layout_create_child_meta (MxBoxLayout  *box,
+                                 ClutterActor *actor)
+{
+  ClutterContainer *container = CLUTTER_CONTAINER (box);
+  ClutterContainerIface *iface = CLUTTER_CONTAINER_GET_IFACE (container);
+
+  g_assert (g_type_is_a (iface->child_meta_type, MX_TYPE_BOX_LAYOUT_CHILD));
+
+  if (G_LIKELY (iface->create_child_meta))
+      iface->create_child_meta (container, actor);
+}
+
+/**
+ * mx_box_layout_add_actor:
+ * @box: a #MxBoxLayout
+ * @actor: the #ClutterActor actor to add to the box layout
+ * @position: the position where to insert the actor
+ *
+ * Inserts @actor at @position in @box.
+ */
+void
+mx_box_layout_add_actor (MxBoxLayout  *box,
+                         ClutterActor *actor,
+                         gint          position)
+{
+  MxBoxLayoutPrivate *priv;
+
+  g_return_if_fail (MX_IS_BOX_LAYOUT (box));
+  g_return_if_fail (CLUTTER_IS_ACTOR (actor));
+
+  priv = box->priv;
+
+  /* this is really mx_box_container_add_actor() with a different insert() */
+  priv->children = g_list_insert (priv->children,
+                                  actor,
+                                  position);
+  mx_box_layout_create_child_meta (box, actor);
+  clutter_actor_set_parent (actor, (ClutterActor*) box);
+
+  if (priv->enable_animations)
+    {
+      _mx_box_layout_start_animation (box);
+
+      if (priv->timeline)
+        {
+          /* fade in the new actor when there is room */
+          clutter_actor_set_opacity (actor, 0);
+          g_signal_connect_swapped (priv->timeline, "completed",
+                                    G_CALLBACK (fade_in_actor), actor);
+        }
+    }
+  else
+    {
+      clutter_actor_queue_relayout ((ClutterActor *) box);
+    }
+
+  g_signal_emit_by_name (box, "actor-added", actor);
+
+}
+
+/**
+ * mx_box_layout_add_actor_with_properties:
+ * @box: a #MxBoxLayout
+ * @actor: the #ClutterActor actor to add to the box layout
+ * @position: the position where to insert the actor
+ * @first_property: name of the first property to set
+ * @...: value for the first property, followed optionally by more name/value
+ *       pairs terminated with NULL.
+ *
+ * Inserts @actor at @position in the layout @box. You can set some layout
+ * properties on the child at the same time.
+ *
+ * If @position is negative, or is larger than the number of actors in the
+ * layout, the new actor is added on to the end of the list.
+ */
+void
+mx_box_layout_add_actor_with_properties (MxBoxLayout  *box,
+                                         ClutterActor *actor,
+                                         gint          position,
+                                         const char   *first_property,
+                                         ...)
+{
+  va_list var_args;
+
+  mx_box_layout_add_actor (box, actor, position);
+
+  if (first_property == NULL || *first_property == '\0')
+    return;
+
+  va_start (var_args, first_property);
+  mx_box_layout_set_property_valist (box, actor, first_property, var_args);
+  va_end (var_args);
+}
