@@ -145,8 +145,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
 static MxFocusable*
 mx_button_accept_focus (MxFocusable *focusable, MxFocusHint hint)
 {
-  /* FIXME: pretend hover is the same as focus for now */
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (focusable), "hover");
+  mx_stylable_set_style_pseudo_class (MX_STYLABLE (focusable), "focus");
   MX_BUTTON (focusable)->priv->is_hover = TRUE;
 
   clutter_actor_grab_key_focus (CLUTTER_ACTOR (focusable));
@@ -235,18 +234,6 @@ mx_button_style_changed (MxWidget *widget)
       g_boxed_free (MX_TYPE_BORDER_IMAGE, content_image);
       g_free (icon_name);
 
-      if (icon_name)
-        g_warning ("The \"-mx-icon-name\" property is ignored when"
-                   " \"-mx-content-image\" is set");
-
-      if (mx_widget_get_background_image (widget))
-        g_warning ("The \"background-image\" property is ignored when"
-                   " \"-mx-content-image\" is set");
-
-      if (mx_widget_get_border_image (widget))
-        g_warning ("The \"border-image\" property is ignored when"
-                   " \"-mx-content-image\" is set");
-
       return;
     }
 
@@ -801,6 +788,9 @@ mx_button_set_label (MxButton    *button,
                             "ellipsize", PANGO_ELLIPSIZE_END,
                             NULL);
       mx_bin_set_child ((MxBin*) button, label);
+      if (CLUTTER_ACTOR_IS_MAPPED (button))
+        mx_stylable_apply_clutter_text_attributes (MX_STYLABLE (button),
+                                                   CLUTTER_TEXT (label));
     }
 
   g_object_notify (G_OBJECT (button), "label");
