@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-entry.c: Plain entry actor
  *
@@ -63,6 +64,7 @@
 #include "mx-texture-cache.h"
 #include "mx-marshal.h"
 #include "mx-clipboard.h"
+#include "mx-focusable.h"
 
 /* for pointer cursor support */
 #include <clutter/x11/clutter-x11.h>
@@ -77,8 +79,10 @@ enum
 {
   PROP_0,
 
-  PROP_ENTRY,
-  PROP_HINT
+  PROP_CLUTTER_TEXT,
+  PROP_HINT_TEXT,
+  PROP_TEXT,
+  PROP_PASSWORD_CHAR
 };
 
 /* signals */
@@ -105,15 +109,23 @@ struct _MxEntryPrivate
   gfloat        spacing;
 
   gboolean hint_visible;
+  gunichar password_char;
+
+  gboolean  pause_undo;
+  GQueue   *undo_history;
+  gulong    undo_timeout_source;
 };
 
 static guint entry_signals[LAST_SIGNAL] = { 0, };
 
 static void mx_stylable_iface_init (MxStylableIface *iface);
+static void mx_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxEntry, mx_entry, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                mx_stylable_iface_init));
+                                                mx_stylable_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_focusable_iface_init));
 
 static void
 mx_entry_set_property (GObject      *gobject,
@@ -125,12 +137,16 @@ mx_entry_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case PROP_ENTRY:
+    case PROP_HINT_TEXT:
+      mx_entry_set_hint_text (entry, g_value_get_string (value));
+      break;
+
+    case PROP_TEXT:
       mx_entry_set_text (entry, g_value_get_string (value));
       break;
 
-    case PROP_HINT:
-      mx_entry_set_hint_text (entry, g_value_get_string (value));
+    case PROP_PASSWORD_CHAR:
+      mx_entry_set_password_char (entry, g_value_get_uint (value));
       break;
 
     default:
@@ -149,12 +165,22 @@ mx_entry_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case PROP_ENTRY:
-      g_value_set_string (value, clutter_text_get_text (CLUTTER_TEXT (priv->entry)));
+    case PROP_CLUTTER_TEXT:
+      g_value_set_object (value, priv->entry);
       break;
 
-    case PROP_HINT:
+    case PROP_HINT_TEXT:
       g_value_set_string (value, priv->hint);
+      break;
+
+    case PROP_TEXT:
+      g_value_set_string (value,
+                          clutter_text_get_text (CLUTTER_TEXT (priv->entry)));
+      break;
+
+    case PROP_PASSWORD_CHAR:
+      g_value_set_uint (value, priv->password_char);
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
@@ -184,6 +210,19 @@ mx_entry_finalize (GObject *object)
   g_free (priv->hint);
   priv->hint = NULL;
 
+  if (priv->undo_history)
+    {
+      g_queue_foreach (priv->undo_history, (GFunc) g_free, NULL);
+      g_queue_free (priv->undo_history);
+      priv->undo_history = NULL;
+    }
+
+  if (priv->undo_timeout_source)
+    {
+      g_source_remove (priv->undo_timeout_source);
+      priv->undo_timeout_source = 0;
+    }
+
   G_OBJECT_CLASS (mx_entry_parent_class)->finalize (object);
 }
 
@@ -216,6 +255,20 @@ mx_stylable_iface_init (MxStylableIface *iface)
     }
 }
 
+static MxFocusable*
+mx_entry_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  clutter_actor_grab_key_focus (MX_ENTRY (focusable)->priv->entry);
+
+  return focusable;
+}
+
+static void
+mx_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->accept_focus = mx_entry_accept_focus;
+}
+
 static void
 mx_entry_style_changed (MxWidget *self)
 {
@@ -283,6 +336,7 @@ mx_entry_get_preferred_width (ClutterActor *actor,
   MxEntryPrivate *priv = MX_ENTRY_PRIV (actor);
   MxPadding padding;
   gfloat icon_w;
+  gfloat entry_height;
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
 
@@ -291,6 +345,12 @@ mx_entry_get_preferred_width (ClutterActor *actor,
   clutter_actor_get_preferred_width (priv->entry, for_height,
                                      min_width_p,
                                      natural_width_p);
+  clutter_actor_get_preferred_height (priv->entry, -1,
+                                      NULL, &entry_height);
+
+  /* ensure the preferred width is at least large enough to be useful */
+  if (natural_width_p)
+    *natural_width_p = MAX ((entry_height * 6), *natural_width_p);
 
   if (priv->primary_icon)
     {
@@ -458,6 +518,11 @@ clutter_text_focus_in_cb (ClutterText  *text,
       priv->hint_visible = FALSE;
 
       clutter_text_set_text (text, "");
+
+      if (priv->password_char != 0)
+        {
+          clutter_text_set_password_char (text, priv->password_char);
+        }
     }
   mx_stylable_set_style_pseudo_class (MX_STYLABLE (actor), "focus");
   clutter_text_set_cursor_visible (text, TRUE);
@@ -476,6 +541,11 @@ clutter_text_focus_out_cb (ClutterText  *text,
 
       clutter_text_set_text (text, priv->hint);
       mx_stylable_set_style_pseudo_class (MX_STYLABLE (actor), "indeterminate");
+
+      if (clutter_text_get_password_char (text) != 0)
+        {
+          clutter_text_set_password_char (text, 0);
+        }
     }
   else
     {
@@ -510,6 +580,9 @@ mx_entry_pick (ClutterActor       *actor,
 
   CLUTTER_ACTOR_CLASS (mx_entry_parent_class)->pick (actor, c);
 
+  if (!clutter_actor_should_pick_paint (actor))
+    return;
+
   clutter_actor_paint (priv->entry);
 
   if (priv->primary_icon)
@@ -633,6 +706,31 @@ mx_entry_key_press_event (ClutterActor    *actor,
       return TRUE;
     }
 
+  /* undo */
+  if ((event->modifier_state & CLUTTER_CONTROL_MASK)
+      && event->keyval == CLUTTER_z)
+    {
+      gchar *str;
+
+      /* pop the current value off the undo history stack and set the text to
+       * the new value */
+      if (priv->undo_timeout_source == 0)
+        {
+          str = g_queue_pop_head (priv->undo_history);
+          g_free (str);
+        }
+
+      /* prevent storing the value just restored */
+      priv->pause_undo = TRUE;
+
+      if (priv->undo_history)
+        {
+          clutter_text_set_text (CLUTTER_TEXT (priv->entry),
+                                 (gchar*) g_queue_peek_head (priv->undo_history));
+        }
+      return TRUE;
+    }
+
   return FALSE;
 }
 
@@ -718,18 +816,31 @@ mx_entry_class_init (MxEntryClass *klass)
   actor_class->key_press_event = mx_entry_key_press_event;
   actor_class->key_focus_in = mx_entry_key_focus_in;
 
-  pspec = g_param_spec_string ("text",
-                               "Text",
-                               "Text of the entry",
-                               NULL, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_ENTRY, pspec);
+  pspec = g_param_spec_object ("clutter-text",
+			       "Clutter Text",
+			       "Internal ClutterText actor",
+			       CLUTTER_TYPE_TEXT,
+			       G_PARAM_READABLE);
+  g_object_class_install_property (gobject_class, PROP_CLUTTER_TEXT, pspec);
 
   pspec = g_param_spec_string ("hint-text",
                                "Hint Text",
                                "Text to display when the entry is not focused "
                                "and the text property is empty",
                                NULL, G_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_ENTRY, pspec);
+  g_object_class_install_property (gobject_class, PROP_HINT_TEXT, pspec);
+
+  pspec = g_param_spec_string ("text",
+                               "Text",
+                               "Text of the entry",
+                               NULL, G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_TEXT, pspec);
+
+  pspec = g_param_spec_unichar ("password-char",
+                                "Password Character",
+                                "Character to display instead of entered text",
+                                0, G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_PASSWORD_CHAR, pspec);
 
   /* signals */
   /**
@@ -760,6 +871,94 @@ mx_entry_class_init (MxEntryClass *klass)
                   G_TYPE_NONE, 0);
 }
 
+
+static gboolean
+mx_entry_store_undo_timeout (MxEntry *entry)
+{
+  const gchar *str;
+  MxEntryPrivate *priv = entry->priv;
+
+  priv->undo_timeout_source = 0;
+
+  str = mx_entry_get_text (entry);
+
+  if (!priv->undo_history)
+    priv->undo_history = g_queue_new ();
+
+  /* prevent duplicated */
+  if (!g_strcmp0 (str, (gchar*) g_queue_peek_head (priv->undo_history)))
+    {
+      return FALSE;
+    }
+
+  g_queue_push_head (priv->undo_history, g_strdup (str));
+
+  /* keep the undo history to only 20 items */
+  if (g_queue_get_length (priv->undo_history) > 20)
+    {
+      gpointer *tmp;
+
+      tmp = g_queue_pop_tail (priv->undo_history);
+
+      g_free (tmp);
+    }
+
+  return FALSE;
+}
+
+static void
+mx_entry_store_undo_history (ClutterText *text,
+                             MxEntry     *entry)
+{
+  if (entry->priv->undo_timeout_source)
+    {
+      g_source_remove (entry->priv->undo_timeout_source);
+      entry->priv->undo_timeout_source = 0;
+    }
+
+  if (entry->priv->pause_undo)
+    {
+      entry->priv->pause_undo = FALSE;
+      return;
+    }
+
+  entry->priv->undo_timeout_source =
+    g_timeout_add (750, (GSourceFunc) mx_entry_store_undo_timeout, entry);
+}
+
+static gboolean
+mx_entry_store_undo_on_keypress (ClutterText     *text,
+                                 ClutterKeyEvent *event,
+                                 MxEntry         *entry)
+{
+  if (event->keyval == CLUTTER_space)
+    {
+      /* store the text immediately */
+      if (entry->priv->undo_timeout_source)
+        {
+          g_source_remove (entry->priv->undo_timeout_source);
+          entry->priv->undo_timeout_source = 0;
+        }
+
+      mx_entry_store_undo_timeout (entry);
+
+      entry->priv->pause_undo = TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_text_changed_cb (ClutterText *text,
+                         MxEntry     *entry)
+{
+  g_return_if_fail (MX_IS_ENTRY (entry));
+
+  g_object_notify (G_OBJECT (entry), "text");
+
+  mx_entry_store_undo_history (text, entry);
+}
+
 static void
 mx_entry_init (MxEntry *entry)
 {
@@ -767,7 +966,7 @@ mx_entry_init (MxEntry *entry)
 
   priv = entry->priv = MX_ENTRY_GET_PRIVATE (entry);
 
-#ifdef HAVE_CLUTTER_IM_CONTEXT
+#ifdef HAVE_CLUTTER_IMCONTEXT
   priv->entry = g_object_new (CLUTTER_TYPE_IMTEXT,
 #else
   priv->entry = g_object_new (CLUTTER_TYPE_TEXT,
@@ -784,6 +983,13 @@ mx_entry_init (MxEntry *entry)
   g_signal_connect (priv->entry, "key-focus-out",
                     G_CALLBACK (clutter_text_focus_out_cb), entry);
 
+  g_signal_connect (priv->entry, "text-changed",
+                    G_CALLBACK (clutter_text_changed_cb), entry);
+
+  g_signal_connect (priv->entry, "key-press-event",
+                    G_CALLBACK (mx_entry_store_undo_on_keypress), entry);
+
+
   priv->spacing = 6.0f;
 
   clutter_actor_set_parent (priv->entry, CLUTTER_ACTOR (entry));
@@ -798,6 +1004,20 @@ mx_entry_init (MxEntry *entry)
 
 /**
  * mx_entry_new:
+ *
+ * Create a new #MxEntry
+ *
+ * Returns: a new #MxEntry
+ */
+ClutterActor *
+mx_entry_new (void)
+{
+  return g_object_new (MX_TYPE_ENTRY, NULL);
+}
+
+
+/**
+ * mx_entry_new_with_text:
  * @text: text to set the entry to
  *
  * Create a new #MxEntry with the specified entry
@@ -805,11 +1025,10 @@ mx_entry_init (MxEntry *entry)
  * Returns: a new #MxEntry
  */
 ClutterActor *
-mx_entry_new (const gchar *text)
+mx_entry_new_with_text (const gchar *text)
 {
   MxWidget *entry;
 
-  /* add the entry to the stage, but don't allow it to be visible */
   entry = g_object_new (MX_TYPE_ENTRY,
                         "text", text,
                         NULL);
@@ -848,6 +1067,7 @@ mx_entry_set_text (MxEntry     *entry,
                    const gchar *text)
 {
   MxEntryPrivate *priv;
+  gunichar password_char;
 
   g_return_if_fail (MX_IS_ENTRY (entry));
 
@@ -861,6 +1081,7 @@ mx_entry_set_text (MxEntry     *entry,
       text = priv->hint;
       priv->hint_visible = TRUE;
       mx_stylable_set_style_pseudo_class (MX_STYLABLE (entry), "indeterminate");
+      password_char = 0;
     }
   else
     {
@@ -870,11 +1091,14 @@ mx_entry_set_text (MxEntry     *entry,
         mx_stylable_set_style_pseudo_class (MX_STYLABLE (entry), NULL);
 
       priv->hint_visible = FALSE;
+      password_char = priv->password_char;
     }
 
   clutter_text_set_text (CLUTTER_TEXT (priv->entry), text);
-
-  g_object_notify (G_OBJECT (entry), "text");
+  if (clutter_text_get_password_char (CLUTTER_TEXT (priv->entry)) != password_char)
+    {
+      clutter_text_set_password_char (CLUTTER_TEXT (priv->entry), password_char);
+    }
 }
 
 /**
@@ -883,8 +1107,8 @@ mx_entry_set_text (MxEntry     *entry,
  *
  * Retrieve the internal #ClutterText so that extra parameters can be set
  *
- * Returns: the #ClutterText used by #MxEntry. The entry is owned by the
- * #MxEntry and should not be unref'ed by the application.
+ * Returns: (transfer none): the #ClutterText used by #MxEntry. The entry is
+ * owned by the #MxEntry and should not be unref'ed by the application.
  */
 ClutterActor*
 mx_entry_get_clutter_text (MxEntry *entry)
@@ -924,6 +1148,10 @@ mx_entry_set_hint_text (MxEntry     *entry,
 
       clutter_text_set_text (CLUTTER_TEXT (priv->entry), priv->hint);
       mx_stylable_set_style_pseudo_class (MX_STYLABLE (entry), "indeterminate");
+      if (clutter_text_get_password_char (CLUTTER_TEXT (priv->entry)) != 0)
+        {
+          clutter_text_set_password_char (CLUTTER_TEXT (priv->entry), 0);
+        }
     }
 }
 
@@ -945,6 +1173,49 @@ mx_entry_get_hint_text (MxEntry *entry)
   return entry->priv->hint;
 }
 
+/**
+ * mx_entry_set_password_char:
+ * @entry: a #MxEntry
+ * @password_char: text to set as the entry hint
+ *
+ * Sets the character to display instead of the text. Use 0 to display
+ * the actual text.
+ */
+void
+mx_entry_set_password_char (MxEntry  *entry,
+                            gunichar  password_char)
+{
+  MxEntryPrivate *priv;
+
+  g_return_if_fail (MX_IS_ENTRY (entry));
+
+  priv = entry->priv;
+
+  priv->password_char = password_char;
+
+  if (!priv->hint_visible)
+    {
+      clutter_text_set_password_char (CLUTTER_TEXT (priv->entry),
+                                      password_char);
+    }
+}
+
+/**
+ * mx_entry_get_password_char:
+ * @entry: a #MxEntry
+ *
+ * Gets the character to display instead of the text.
+ *
+ * Return value: a character, or 0 if input should not be hidden.
+ */
+gunichar
+mx_entry_get_password_char (MxEntry *entry)
+{
+  g_return_val_if_fail (MX_IS_ENTRY (entry), 0);
+
+  return entry->priv->password_char;
+}
+
 static gboolean
 _mx_entry_icon_press_cb (ClutterActor       *actor,
                          ClutterButtonEvent *event,
