@@ -1,7 +1,8 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-expander.c: Expander Widget
  *
- * Copyright 2009 Intel Corporation.
+ * Copyright 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -85,7 +86,7 @@ struct _MxExpanderPrivate {
   ClutterAlpha    *alpha;
   gdouble          progress;
 
-  gboolean         expanded : 1;
+  guint            expanded : 1;
 };
 
 static guint expander_signals[LAST_SIGNAL] = { 0, };
@@ -178,6 +179,15 @@ timeline_complete (ClutterTimeline *timeline,
 
   g_signal_emit (expander, expander_signals[EXPAND_COMPLETE], 0);
 
+  /* if the expander is now closed, update the style */
+  if (!priv->expanded)
+    {
+      clutter_actor_set_name (priv->arrow, "mx-expander-arrow-closed");
+      mx_stylable_set_style_class (MX_STYLABLE (expander), "closed-expander");
+
+      clutter_actor_queue_relayout (expander);
+    }
+
   child = mx_bin_get_child (MX_BIN (expander));
 
   if (!child)
@@ -217,24 +227,17 @@ new_frame (ClutterTimeline *timeline,
 }
 
 static void
-mx_expander_toggle_expanded (MxExpander *expander)
+mx_expander_update (MxExpander *expander)
 {
   MxExpanderPrivate *priv = expander->priv;
   ClutterActor *child;
 
-  priv->expanded = !priv->expanded;
-  g_object_notify ((GObject *) expander, "expanded");
-
   if (priv->expanded)
     {
       clutter_actor_set_name (priv->arrow, "mx-expander-arrow-open");
-      mx_stylable_set_style_pseudo_class (MX_STYLABLE (expander), "active");
-    }
-  else
-    {
-      clutter_actor_set_name (priv->arrow, "mx-expander-arrow-closed");
-      mx_stylable_set_style_pseudo_class (MX_STYLABLE (expander), NULL);
+      mx_stylable_set_style_class (MX_STYLABLE (expander), "open-expander");
     }
+  /* closed state is set when animation is finished */
 
   child = mx_bin_get_child (MX_BIN (expander));
 
@@ -265,7 +268,9 @@ static gboolean
 mx_expander_button_release (ClutterActor       *actor,
                             ClutterButtonEvent *event)
 {
-  mx_expander_toggle_expanded (MX_EXPANDER (actor));
+  MxExpander *expander = MX_EXPANDER (actor);
+
+  mx_expander_set_expanded (expander, !expander->priv->expanded);
 
   return FALSE;
 }
@@ -387,13 +392,11 @@ mx_expander_allocate (ClutterActor          *actor,
   MxPadding padding;
   gfloat label_w, label_h;
   gfloat child_h, child_w;
-  ClutterActorClass *parent_parent;
   gfloat available_w, available_h, min_w, min_h, full_h, arrow_h, arrow_w;
   ClutterRequestMode request;
 
-  /* skip MxBin allocate */
-  parent_parent = g_type_class_peek_parent (mx_expander_parent_class);
-  parent_parent->allocate (actor, box, flags);
+  /* chain up to store allocation */
+  CLUTTER_ACTOR_CLASS (mx_expander_parent_class)->allocate (actor, box, flags);
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
 
@@ -432,7 +435,9 @@ mx_expander_allocate (ClutterActor          *actor,
   child_box.x1 = padding.left + arrow_w + 6.0f;
   child_box.x2 = child_box.x1 + label_w;
   child_box.y1 = padding.top;
-  child_box.y2 = child_box.y1 + label_h;
+  child_box.y2 = child_box.y1 + MAX (label_h, arrow_h);
+  mx_allocate_align_fill (priv->label, &child_box, MX_ALIGN_START,
+                          MX_ALIGN_MIDDLE, FALSE, FALSE);
   clutter_actor_allocate (priv->label, &child_box, flags);
 
   full_h += MAX (label_h, arrow_h);
@@ -470,7 +475,7 @@ mx_expander_allocate (ClutterActor          *actor,
       child_box.x1 = padding.left;
       child_box.x2 = child_box.x1 + child_w;
       child_box.y1 = padding.top + priv->spacing + MAX (label_h, arrow_h);
-      child_box.y2 = child_box.y2 + child_h;
+      child_box.y2 = child_box.y1 + child_h;
       clutter_actor_allocate (child, &child_box, flags);
 
       full_h += priv->spacing + child_h;
@@ -511,73 +516,19 @@ mx_expander_unmap (ClutterActor *actor)
 }
 
 static void
-mx_expander_style_changed (MxWidget *widget)
+mx_expander_style_changed (MxStylable *stylable)
 {
-  MxExpander *expander = MX_EXPANDER (widget);
+  MxExpander *expander = MX_EXPANDER (stylable);
   MxExpanderPrivate *priv = expander->priv;
   const gchar *pseudo_class;
-  ClutterColor *color = NULL;
-  gchar *font_name;
-  gchar *font_string;
-  gint font_size;
 
-  pseudo_class = mx_stylable_get_style_pseudo_class (MX_STYLABLE (widget));
+  pseudo_class = mx_stylable_get_style_pseudo_class (stylable);
 
   mx_stylable_set_style_pseudo_class (MX_STYLABLE (expander->priv->arrow),
-                                    pseudo_class);
+                                      pseudo_class);
 
-
-  mx_stylable_get (MX_STYLABLE (widget),
-                   "color", &color,
-                   "font-family", &font_name,
-                   "font-size", &font_size,
-                   NULL);
-
-  if (color)
-    {
-      clutter_text_set_color (CLUTTER_TEXT (priv->label), color);
-      clutter_color_free (color);
-    }
-
-  if (font_name || font_size)
-    {
-      if (font_name && font_size)
-        {
-          font_string = g_strdup_printf ("%s %dpx", font_name, font_size);
-          g_free (font_name);
-        }
-      else
-        {
-          if (font_size)
-            font_string = g_strdup_printf ("%dpx", font_size);
-          else
-            font_string = font_name;
-        }
-
-      clutter_text_set_font_name (CLUTTER_TEXT (priv->label), font_string);
-      g_free (font_string);
-    }
-
-}
-
-static gboolean
-mx_expander_enter (ClutterActor         *actor,
-                   ClutterCrossingEvent *event)
-{
-  if (!MX_EXPANDER (actor)->priv->expanded)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (actor), "hover");
-
-  return FALSE;
-}
-
-static gboolean
-mx_expander_leave (ClutterActor         *actor,
-                   ClutterCrossingEvent *event)
-{
-  if (!MX_EXPANDER (actor)->priv->expanded)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (actor), NULL);
-
-  return FALSE;
+  mx_stylable_apply_clutter_text_attributes (stylable,
+                                             CLUTTER_TEXT (priv->label));
 }
 
 static void
@@ -585,11 +536,12 @@ mx_expander_foreach (ClutterContainer *container,
                      ClutterCallback   callback,
                      gpointer          user_data)
 {
-  MxExpanderPrivate *priv = MX_EXPANDER (container)->priv;
+  ClutterActor *child;
 
-  if (priv->expanded)
-    callback (mx_bin_get_child ((MxBin*) container),
-              user_data);
+  child = mx_bin_get_child (MX_BIN (container));
+
+  if (child)
+    callback (child, user_data);
 }
 
 static void
@@ -641,8 +593,6 @@ mx_expander_class_init (MxExpanderClass *klass)
   actor_class->get_preferred_width = mx_expander_get_preferred_width;
   actor_class->get_preferred_height = mx_expander_get_preferred_height;
   actor_class->paint = mx_expander_paint;
-  actor_class->enter_event = mx_expander_enter;
-  actor_class->leave_event = mx_expander_leave;
   actor_class->map = mx_expander_map;
   actor_class->unmap = mx_expander_unmap;
 
@@ -755,7 +705,11 @@ mx_expander_set_expanded (MxExpander *expander,
 
   if (expander->priv->expanded != expanded)
     {
-      mx_expander_toggle_expanded (expander);
+      expander->priv->expanded = expanded;
+
+      mx_expander_update (expander);
+
+      g_object_notify (G_OBJECT (expander), "expanded");
     }
 }
 
