@@ -1,7 +1,8 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-grid.c: Reflowing grid layout container for mx.
  *
- * Copyright 2008, 2009 Intel Corporation.
+ * Copyright 2008, 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -46,7 +47,7 @@
  *   of the contained actors (#MxGrid:homogenous_columns and #MxGrid:homogenous_rows)
  *   </para></listitem>
  *   <listitem><para>Prefer to pack children vertically first,
- *   rather than horizontally (#MxGrid:vertical)</para></listitem>
+ *   rather than horizontally (#MxGrid:orientation)</para></listitem>
  *   <listitem><para>Specify the maximum number of rows or columns 
  *   to allow in the layout, to prevent it from being excessively stretched 
  *   (#MxGrid:max_stride).</para></listitem>
@@ -57,7 +58,7 @@
  * <figure id="mx-grid">
  *   <title>MxGrid flowing across multiple rows</title>
  *   <para>An #MxGrid containing 9 child actors; 
- *   #MxGrid:vertical is set to the default (#FALSE, 
+ *   #MxGrid:orientation is set to the default (MX_ORIENTATION_HORIZONTAL, 
  *   i.e. lay out horizontally first); #MxGrid:max_stride has not been set 
  *   (so there's no maximum row size); #MxGrid:column_spacing and #MxGrid:row_spacing have
  *   been set so that there is spacing between cells vertically and horizontally.</para>
@@ -77,7 +78,7 @@
  *   how the "odd" rectangle is on the end of a row, rather than at the 
  *   bottom of a column. This is because preference 
  *   is being given to packing onto the end of rows, rather than columns, 
- *   because #MxGrid:vertical is set to #FALSE. Even though 
+ *   because #MxGrid:orientation is set to MX_ORIENTATION_HORIZONTAL. Even though 
  *   there is room for the rectangle at the bottom of the column, the 
  *   layout prefers to place children onto the end of a row if there is room.</para>
  *   <graphic fileref="MxGrid-2rows-row-major.png" format="PNG"/>
@@ -85,11 +86,11 @@
  *
  * <figure id="mx-grid-two-columns">
  *   <title>MxGrid flowing into two columns</title>
- *   <para>The same #MxGrid 9 children with #MxGrid:vertical set to #TRUE. This time,
- *   the layout wraps onto two columns rather than two rows. Even though 
- *   there is room on the end of the rows for the children, the preference 
- *   is for them to be placed on the bottom of columns, or into new columns, 
- *   before being added to rows.</para>
+ *   <para>The same #MxGrid 9 children with #MxGrid:orientation set to
+ *   MX_ORIENTATION_VERTICAL. This time, the layout wraps onto two columns rather than two
+ *   rows. Even though there is room on the end of the rows for the children,
+ *   the preference is for them to be placed on the bottom of columns, or into
+ *   new columns, before being added to rows.</para>
  *   <graphic fileref="MxGrid-2cols-column-major.png" format="PNG"/>
  * </figure>
  */
@@ -99,6 +100,9 @@
 #include "mx-scrollable.h"
 #include "mx-grid.h"
 #include "mx-stylable.h"
+#include "mx-focusable.h"
+#include "mx-enum-types.h"
+#include "mx-private.h"
 
 typedef struct _MxGridActorData MxGridActorData;
 
@@ -164,14 +168,17 @@ mx_grid_do_allocate (ClutterActor          *self,
                      gfloat                *actual_width,
                      gfloat                *actual_height);
 
-static void scrollable_interface_init (MxScrollableInterface *iface);
+static void scrollable_interface_init (MxScrollableIface *iface);
+static void mx_box_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxGrid, mx_grid,
                          MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
                                                 clutter_container_iface_init)
                          G_IMPLEMENT_INTERFACE (MX_TYPE_SCROLLABLE,
-                                                scrollable_interface_init));
+                                                scrollable_interface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_box_focusable_iface_init));
 
 #define MX_GRID_GET_PRIVATE(obj) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), MX_TYPE_GRID, \
@@ -184,11 +191,12 @@ struct _MxGridPrivate
 
   gboolean      homogenous_rows;
   gboolean      homogenous_columns;
-  gboolean      end_align;
+  MxAlign       line_alignment;
   gfloat        column_spacing, row_spacing;
-  gdouble       valign, halign;
+  MxAlign       child_x_align;
+  MxAlign       child_y_align;
 
-  gboolean      vertical;
+  MxOrientation orientation;
 
   gboolean      first_of_batch;
   gfloat        a_current_sum, a_wrap;
@@ -200,7 +208,7 @@ struct _MxGridPrivate
   MxAdjustment *hadjustment;
   MxAdjustment *vadjustment;
 
-  gboolean      allocate_hidden;
+  MxFocusable  *last_focus;
 };
 
 enum
@@ -210,14 +218,13 @@ enum
   PROP_HOMOGENOUS_COLUMNS,
   PROP_ROW_SPACING,
   PROP_COLUMN_SPACING,
-  PROP_VALIGN,
-  PROP_HALIGN,
-  PROP_END_ALIGN,
-  PROP_VERTICAL,
+  PROP_CHILD_X_ALIGN,
+  PROP_CHILD_Y_ALIGN,
+  PROP_LINE_ALIGNMENT,
+  PROP_ORIENTATION,
   PROP_HADJUST,
   PROP_VADJUST,
   PROP_MAX_STRIDE,
-  PROP_ALLOCATE_HIDDEN
 };
 
 struct _MxGridActorData
@@ -227,66 +234,12 @@ struct _MxGridActorData
   gfloat   pref_width, pref_height;
 };
 
-static void
-ensure_children_are_visible (MxGrid *grid)
-{
-  MxGridPrivate *priv = grid->priv;
-  GList *child_item;
-  gfloat x, y;
-  ClutterActorBox grid_b;
-
-  if (priv->hadjustment)
-    x = mx_adjustment_get_value (priv->hadjustment);
-  else
-    x = 0;
-
-  if (priv->vadjustment)
-    y = mx_adjustment_get_value (priv->vadjustment);
-  else
-    y = 0;
-
-  clutter_actor_get_allocation_box ((ClutterActor *) grid, &grid_b);
-  grid_b.x2 = (grid_b.x2 - grid_b.x1) + x;
-  grid_b.x1 = 0;
-  grid_b.y2 = (grid_b.y2 - grid_b.y1) + y;
-  grid_b.y1 = 0;
-
-  for (child_item = priv->list;
-       child_item != NULL;
-       child_item = child_item->next)
-    {
-      ClutterActor *child = child_item->data;
-      ClutterActorBox child_b;
-
-      g_assert (child != NULL);
-
-      /* check if the child is "on screen" */
-      clutter_actor_get_allocation_box (child, &child_b);
-
-      if ((child_b.x1 < grid_b.x2)
-          && (child_b.x2 > grid_b.x1)
-          && (child_b.y1 < grid_b.y2)
-          && (child_b.y2 > grid_b.y1))
-        {
-          clutter_actor_show (child);
-        } else {
-          clutter_actor_hide (child);
-        }
-    }
-}
-
 /* scrollable interface */
 static void
 adjustment_value_notify_cb (MxAdjustment *adjustment,
                             GParamSpec   *pspec,
                             MxGrid       *grid)
 {
-  MxGridPrivate *priv = grid->priv;
-
-  /* Mark children as visible if they are inside the visible area */
-  if (priv->allocate_hidden)
-    ensure_children_are_visible (grid);
-
   clutter_actor_queue_redraw (CLUTTER_ACTOR (grid));
 }
 
@@ -362,17 +315,14 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
           /* create an initial adjustment. this is filled with correct values
            * as soon as allocate() is called */
 
-          adjustment = mx_adjustment_new (0.0,
-                                          0.0,
-                                          1.0,
-                                          1.0,
-                                          1.0,
-                                          1.0);
+          adjustment = mx_adjustment_new ();
 
           scrollable_set_adjustments (scrollable,
                                       adjustment,
                                       priv->vadjustment);
 
+          g_object_unref (adjustment);
+
           *hadjustment = adjustment;
         }
     }
@@ -388,24 +338,21 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
           /* create an initial adjustment. this is filled with correct values
            * as soon as allocate() is called */
 
-          adjustment = mx_adjustment_new (0.0,
-                                          0.0,
-                                          1.0,
-                                          1.0,
-                                          1.0,
-                                          1.0);
+          adjustment = mx_adjustment_new ();
 
           scrollable_set_adjustments (scrollable,
                                       priv->hadjustment,
                                       adjustment);
 
+          g_object_unref (adjustment);
+
           *vadjustment = adjustment;
         }
     }
 }
 
 static void
-scrollable_interface_init (MxScrollableInterface *iface)
+scrollable_interface_init (MxScrollableIface *iface)
 {
   iface->set_adjustments = scrollable_set_adjustments;
   iface->get_adjustments = scrollable_get_adjustments;
@@ -490,64 +437,57 @@ mx_grid_class_init (MxGridClass *klass)
                                    PROP_HOMOGENOUS_COLUMNS,
                                    pspec);
 
-  pspec = g_param_spec_boolean ("vertical",
-                                "Vertical",
-                                "Pack children vertically (in columns), "
-                                "instead of horizontally (in rows)",
-                                FALSE,
-                                G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
-  g_object_class_install_property (gobject_class, PROP_VERTICAL, pspec);
+  pspec = g_param_spec_enum ("orientation",
+                             "Orientation",
+                             "Pack children vertically (in columns), "
+                             "instead of horizontally (in rows)",
+                             MX_TYPE_ORIENTATION,
+                             MX_ORIENTATION_HORIZONTAL,
+                             G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
+  g_object_class_install_property (gobject_class, PROP_ORIENTATION, pspec);
 
-  pspec = g_param_spec_boolean ("end-align",
-                                "end-align",
-                                "Right/bottom aligned rows/columns",
-                                FALSE,
-                                G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
-  g_object_class_install_property (gobject_class, PROP_END_ALIGN, pspec);
+  pspec = g_param_spec_enum ("line-alignment",
+                             "Line Alignment",
+                             "Alignment of rows/columns",
+                             MX_TYPE_ALIGN,
+                             MX_ALIGN_START,
+                             G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
+  g_object_class_install_property (gobject_class, PROP_LINE_ALIGNMENT, pspec);
 
 
-  pspec = g_param_spec_double ("valign",
-                               "Vertical align",
-                               "Vertical alignment of items within cells",
-                               0.0, 1.0, 0.0,
-                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
-  g_object_class_install_property (gobject_class,PROP_VALIGN, pspec);
+  pspec = g_param_spec_enum ("child-y-align",
+                             "Vertical align",
+                             "Vertical alignment of items within cells",
+                             MX_TYPE_ALIGN, MX_ALIGN_START,
+                             G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
+  g_object_class_install_property (gobject_class,PROP_CHILD_Y_ALIGN, pspec);
 
-  pspec = g_param_spec_double ("halign",
-                               "Horizontal align",
-                               "Horizontal alignment of items within cells",
-                               0.0, 1.0, 0.0,
-                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
-  g_object_class_install_property (gobject_class, PROP_HALIGN, pspec);
+  pspec = g_param_spec_enum ("child-x-align",
+                             "Horizontal align",
+                             "Horizontal alignment of items within cells",
+                             MX_TYPE_ALIGN, MX_ALIGN_START,
+                             G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
+  g_object_class_install_property (gobject_class, PROP_CHILD_X_ALIGN, pspec);
 
   pspec = g_param_spec_int ("max-stride",
                             "Maximum stride",
                             "Maximum number of rows or columns, depending"
-                            " on orientation. For example, if max-stride is set to 3 with"
-                            " vertical FALSE, there"
-                            " will be a maximum of 3 children in a row; if vertical"
-                            " is TRUE, there will be a maximum of 3 children in a column",
+                            " on orientation. For example, if max-stride is set"
+                            " to 3 with orientation MX_ORIENTATION_HORIZONTAL, there will"
+                            " be a maximum of 3 children in a row; if"
+                            " orientation is MX_ORIENTATION_VERTICAL, there will be a"
+                            " maximum of 3 children in a column",
                             0, G_MAXINT, 0,
                             G_PARAM_READWRITE|G_PARAM_CONSTRUCT);
   g_object_class_install_property (gobject_class, PROP_MAX_STRIDE, pspec);
 
   g_object_class_override_property (gobject_class,
                                     PROP_HADJUST,
-                                    "hadjustment");
+                                    "horizontal-adjustment");
 
   g_object_class_override_property (gobject_class,
                                     PROP_VADJUST,
-                                    "vadjustment");
-
-  pspec = g_param_spec_boolean ("allocate-hidden",
-                                "Allocate children that are not visible",
-                                "This flag determines whether hidden children"
-                                " should be treated allocated or whether they"
-                                " should be skipped from the allocation and"
-                                " treated as if they are not present",
-                                FALSE,
-                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
-  g_object_class_install_property (gobject_class, PROP_ALLOCATE_HIDDEN, pspec);
+                                    "vertical-adjustment");
 }
 
 static void
@@ -561,6 +501,173 @@ clutter_container_iface_init (ClutterContainerIface *iface)
   iface->sort_depth_order = mx_grid_real_sort_depth_order;
 }
 
+
+/*
+ * focusable implementation
+ */
+
+static void
+update_adjustments (MxGrid      *self,
+                    MxFocusable *focusable)
+{
+  MxGridPrivate *priv = self->priv;
+  gdouble value, new_value, page_size;
+  ClutterActorBox box = { 0, };
+  clutter_actor_get_allocation_box (CLUTTER_ACTOR (focusable), &box);
+
+  if (priv->vadjustment)
+    {
+      mx_adjustment_get_values (priv->vadjustment,
+                                &value, NULL, NULL, NULL, NULL,
+                                &page_size);
+      if (box.y1 < value)
+        new_value = box.y1;
+      else if (box.y2 > value + page_size)
+        new_value = box.y2 - page_size;
+      else
+        new_value = value;
+      mx_adjustment_interpolate (priv->vadjustment,
+                                 new_value,
+                                 250, CLUTTER_EASE_OUT_CUBIC);
+    }
+  if (priv->hadjustment)
+    {
+      mx_adjustment_get_values (priv->hadjustment,
+                                &value, NULL, NULL, NULL, NULL,
+                                &page_size);
+      if (box.x1 < value)
+        new_value = box.x1;
+      else if (box.x2 > value + page_size)
+        new_value = box.x2 - page_size;
+      else
+        new_value = value;
+      mx_adjustment_interpolate (priv->hadjustment,
+                                 new_value,
+                                 250, CLUTTER_EASE_OUT_CUBIC);
+    }
+}
+
+static MxFocusable*
+mx_grid_move_focus (MxFocusable      *focusable,
+                    MxFocusDirection  direction,
+                    MxFocusable      *from)
+{
+  MxGridPrivate *priv = MX_GRID (focusable)->priv;
+  GList *l, *childlink;
+
+  /* find the current focus */
+  childlink = g_list_find (priv->list, from);
+
+  if (!childlink)
+    return NULL;
+
+  /* find the next widget to focus */
+  if (direction == MX_FOCUS_DIRECTION_NEXT)
+    {
+      for (l = childlink->next; l; l = g_list_next (l))
+        {
+          if (MX_IS_FOCUSABLE (l->data))
+            {
+              MxFocusable *focused;
+
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                   MX_FOCUS_HINT_FIRST);
+
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)l->data;
+                  update_adjustments (MX_GRID (focusable), focused);
+                  return focused;
+                }
+            }
+        }
+
+      /* no next widgets to focus */
+      return NULL;
+    }
+  else if (direction == MX_FOCUS_DIRECTION_PREVIOUS)
+    {
+      for (l = g_list_previous (childlink); l; l = g_list_previous (l))
+        {
+          if (MX_IS_FOCUSABLE (l->data))
+            {
+              MxFocusable *focused;
+
+              focused = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                   MX_FOCUS_HINT_LAST);
+
+              if (focused)
+                {
+                  priv->last_focus = (MxFocusable *)l->data;
+                  update_adjustments (MX_GRID (focusable), focused);
+                  return focused;
+                }
+            }
+        }
+    }
+
+  return NULL;
+}
+
+static MxFocusable*
+mx_grid_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  MxGridPrivate *priv = MX_GRID (focusable)->priv;
+  MxFocusable *return_focusable;
+  GList* list, *l;
+
+  return_focusable = NULL;
+
+  /* find the first/last focusable widget */
+  switch (hint)
+    {
+    case MX_FOCUS_HINT_LAST:
+      list = g_list_reverse (g_list_copy (priv->list));
+      break;
+
+    case MX_FOCUS_HINT_PRIOR:
+      if (priv->last_focus)
+        {
+          list = g_list_copy (g_list_find (priv->list, priv->last_focus));
+          if (list)
+            break;
+        }
+      /* This intentionally runs into the next case */
+
+    default:
+    case MX_FOCUS_HINT_FIRST:
+      list = g_list_copy (priv->list);
+      break;
+    }
+
+  for (l = list; l; l = g_list_next (l))
+    {
+      if (MX_IS_FOCUSABLE (l->data))
+        {
+          return_focusable = mx_focusable_accept_focus (MX_FOCUSABLE (l->data),
+                                                        hint);
+
+          if (return_focusable)
+            {
+              update_adjustments (MX_GRID (focusable), return_focusable);
+              break;
+            }
+        }
+    }
+
+  g_list_free (list);
+
+  return return_focusable;
+}
+
+static void
+mx_box_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->move_focus = mx_grid_move_focus;
+  iface->accept_focus = mx_grid_accept_focus;
+}
+
+
 static void
 mx_grid_init (MxGrid *self)
 {
@@ -576,11 +683,6 @@ mx_grid_init (MxGrid *self)
                              g_direct_equal,
                              NULL,
                              mx_grid_free_actor_data);
-
-  /* allocate_hidden defaults to FALSE. This is the original MxGrid
-   * behaviour
-   */
-  priv->allocate_hidden = FALSE;
 }
 
 static void
@@ -611,21 +713,20 @@ mx_grid_finalize (GObject *object)
   G_OBJECT_CLASS (mx_grid_parent_class)->finalize (object);
 }
 
-
 void
-mx_grid_set_end_align (MxGrid  *self,
-                       gboolean value)
+mx_grid_set_line_alignment (MxGrid  *self,
+                            MxAlign  value)
 {
   MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  priv->end_align = value;
+  priv->line_alignment = value;
   clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
 }
 
 gboolean
-mx_grid_get_end_align (MxGrid *self)
+mx_grid_get_line_alignment (MxGrid *self)
 {
   MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  return priv->end_align;
+  return priv->line_alignment;
 }
 
 void
@@ -664,19 +765,29 @@ mx_grid_get_homogenous_columns (MxGrid *self)
 
 
 void
-mx_grid_set_vertical (MxGrid  *self,
-                      gboolean value)
+mx_grid_set_orientation (MxGrid *grid,
+                         MxOrientation orientation)
 {
-  MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  priv->vertical = value;
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
+  MxGridPrivate *priv = grid->priv;
+
+  g_return_if_fail (MX_IS_GRID (grid));
+
+  if (priv->orientation != orientation)
+    {
+      priv->orientation = orientation;
+
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (grid));
+
+      g_object_notify (G_OBJECT (grid), "orientation");
+    }
 }
 
-gboolean
-mx_grid_get_vertical (MxGrid *self)
+MxOrientation
+mx_grid_get_orientation (MxGrid *grid)
 {
-  MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  return priv->vertical;
+  g_return_val_if_fail (MX_IS_GRID (grid), MX_ORIENTATION_HORIZONTAL);
+
+  return grid->priv->orientation;
 }
 
 void
@@ -713,40 +824,38 @@ mx_grid_get_row_spacing (MxGrid *self)
   return priv->row_spacing;
 }
 
-
 void
-mx_grid_set_valign (MxGrid *self,
-                    gdouble value)
+mx_grid_set_child_y_align (MxGrid  *self,
+                           MxAlign  value)
 {
   MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  priv->valign = value;
+  priv->child_y_align = value;
   clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
 }
 
-gdouble
-mx_grid_get_valign (MxGrid *self)
+
+MxAlign
+mx_grid_get_child_y_align (MxGrid *self)
 {
   MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  return priv->valign;
+  return priv->child_y_align;
 }
 
-
-
 void
-mx_grid_set_halign (MxGrid *self,
-                    gdouble value)
+mx_grid_set_child_x_align (MxGrid  *self,
+                           MxAlign  value)
 
 {
   MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  priv->halign = value;
+  priv->child_x_align = value;
   clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
 }
 
-gdouble
-mx_grid_get_halign (MxGrid *self)
+MxAlign
+mx_grid_get_child_x_align (MxGrid *self)
 {
   MxGridPrivate *priv = MX_GRID_GET_PRIVATE (self);
-  return priv->halign;
+  return priv->child_x_align;
 }
 
 void
@@ -781,8 +890,8 @@ mx_grid_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_END_ALIGN:
-      mx_grid_set_end_align (grid, g_value_get_boolean (value));
+    case PROP_LINE_ALIGNMENT:
+      mx_grid_set_line_alignment (grid, g_value_get_enum (value));
       break;
     case PROP_HOMOGENOUS_ROWS:
       mx_grid_set_homogenous_rows (grid, g_value_get_boolean (value));
@@ -790,8 +899,8 @@ mx_grid_set_property (GObject      *object,
     case PROP_HOMOGENOUS_COLUMNS:
       mx_grid_set_homogenous_columns (grid, g_value_get_boolean (value));
       break;
-    case PROP_VERTICAL:
-      mx_grid_set_vertical (grid, g_value_get_boolean (value));
+    case PROP_ORIENTATION:
+      mx_grid_set_orientation (grid, g_value_get_enum (value));
       break;
     case PROP_COLUMN_SPACING:
       mx_grid_set_column_spacing (grid, g_value_get_float (value));
@@ -799,11 +908,11 @@ mx_grid_set_property (GObject      *object,
     case PROP_ROW_SPACING:
       mx_grid_set_row_spacing (grid, g_value_get_float (value));
       break;
-    case PROP_VALIGN:
-      mx_grid_set_valign (grid, g_value_get_double (value));
+    case PROP_CHILD_X_ALIGN:
+      mx_grid_set_child_x_align (grid, g_value_get_enum (value));
       break;
-    case PROP_HALIGN:
-      mx_grid_set_halign (grid, g_value_get_double (value));
+    case PROP_CHILD_Y_ALIGN:
+      mx_grid_set_child_y_align (grid, g_value_get_enum (value));
       break;
     case PROP_HADJUST:
       scrollable_set_adjustments (MX_SCROLLABLE (object),
@@ -818,10 +927,6 @@ mx_grid_set_property (GObject      *object,
     case PROP_MAX_STRIDE:
       mx_grid_set_max_stride (grid, g_value_get_int (value));
       break;
-    case PROP_ALLOCATE_HIDDEN:
-      priv->allocate_hidden = g_value_get_boolean (value);
-      clutter_actor_queue_relayout ((ClutterActor *) object);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -849,11 +954,11 @@ mx_grid_get_property (GObject    *object,
     case PROP_HOMOGENOUS_COLUMNS:
       g_value_set_boolean (value, mx_grid_get_homogenous_columns (grid));
       break;
-    case PROP_END_ALIGN:
-      g_value_set_boolean (value, mx_grid_get_end_align (grid));
+    case PROP_LINE_ALIGNMENT:
+      g_value_set_enum (value, mx_grid_get_line_alignment (grid));
       break;
-    case PROP_VERTICAL:
-      g_value_set_boolean (value, mx_grid_get_vertical (grid));
+    case PROP_ORIENTATION:
+      g_value_set_enum (value, mx_grid_get_orientation (grid));
       break;
     case PROP_COLUMN_SPACING:
       g_value_set_float (value, mx_grid_get_column_spacing (grid));
@@ -861,11 +966,11 @@ mx_grid_get_property (GObject    *object,
     case PROP_ROW_SPACING:
       g_value_set_float (value, mx_grid_get_row_spacing (grid));
       break;
-    case PROP_VALIGN:
-      g_value_set_double (value, mx_grid_get_valign (grid));
+    case PROP_CHILD_X_ALIGN:
+      g_value_set_enum (value, mx_grid_get_child_x_align (grid));
       break;
-    case PROP_HALIGN:
-      g_value_set_double (value, mx_grid_get_halign (grid));
+    case PROP_CHILD_Y_ALIGN:
+      g_value_set_enum (value, mx_grid_get_child_y_align (grid));
       break;
     case PROP_HADJUST:
       scrollable_get_adjustments (MX_SCROLLABLE (grid), &adjustment, NULL);
@@ -878,9 +983,6 @@ mx_grid_get_property (GObject    *object,
     case PROP_MAX_STRIDE:
       g_value_set_int (value, mx_grid_get_max_stride (grid));
       break;
-    case PROP_ALLOCATE_HIDDEN:
-      g_value_set_boolean (value, priv->allocate_hidden);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -945,9 +1047,6 @@ mx_grid_real_remove (ClutterContainer *container,
       clutter_actor_queue_relayout (CLUTTER_ACTOR (layout));
 
       g_signal_emit_by_name (container, "actor-removed", actor);
-
-      if (!priv->allocate_hidden && !CLUTTER_ACTOR_IS_VISIBLE (actor))
-        clutter_actor_queue_redraw (CLUTTER_ACTOR (layout));
     }
   priv->list = g_list_remove (priv->list, actor);
 
@@ -965,12 +1064,29 @@ mx_grid_real_foreach (ClutterContainer *container,
   g_list_foreach (priv->list, (GFunc) callback, user_data);
 }
 
+/*
+ * Implementations for raise, lower and sort_by_depth_order are taken from
+ * ClutterBox.
+ */
 static void
 mx_grid_real_raise (ClutterContainer *container,
                     ClutterActor     *actor,
                     ClutterActor     *sibling)
 {
-  /* STUB */
+  MxGridPrivate *priv = MX_GRID (container)->priv;
+
+  priv->list = g_list_remove (priv->list, actor);
+
+  if (sibling == NULL)
+    priv->list = g_list_append (priv->list, actor);
+  else
+    {
+      gint index_ = g_list_index (priv->list, sibling) + 1;
+
+      priv->list = g_list_insert (priv->list, actor, index_);
+    }
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -978,13 +1094,46 @@ mx_grid_real_lower (ClutterContainer *container,
                     ClutterActor     *actor,
                     ClutterActor     *sibling)
 {
-  /* STUB */
+  MxGridPrivate *priv = MX_GRID (container)->priv;
+
+  priv->list = g_list_remove (priv->list, actor);
+
+  if (sibling == NULL)
+    priv->list = g_list_prepend (priv->list, actor);
+  else
+    {
+      gint index_ = g_list_index (priv->list, sibling);
+
+      priv->list = g_list_insert (priv->list, actor, index_);
+    }
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
+}
+
+static gint
+sort_by_depth (gconstpointer a,
+               gconstpointer b)
+{
+  gfloat depth_a = clutter_actor_get_depth ((ClutterActor *) a);
+  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *) b);
+
+  if (depth_a < depth_b)
+    return -1;
+
+  if (depth_a > depth_b)
+    return 1;
+
+  return 0;
 }
 
 static void
 mx_grid_real_sort_depth_order (ClutterContainer *container)
 {
-  /* STUB */
+  MxGridPrivate *priv = MX_GRID (container)->priv;
+
+  priv->list = g_list_sort (priv->list, sort_by_depth);
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -1148,7 +1297,7 @@ compute_row_height (GList         *siblings,
   gboolean homogenous_b;
   gfloat gap;
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
       homogenous_b = priv->homogenous_columns;
       homogenous_a = priv->homogenous_rows;
@@ -1171,7 +1320,7 @@ compute_row_height (GList         *siblings,
                                         NULL, NULL,
                                         &natural_width, &natural_height);
 
-      if (priv->vertical)
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
         {
           gfloat temp = natural_height;
           natural_height = natural_width;
@@ -1211,7 +1360,7 @@ compute_row_start (GList         *siblings,
   gboolean homogenous_b;
   gfloat gap;
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
       homogenous_b = priv->homogenous_columns;
       homogenous_a = priv->homogenous_rows;
@@ -1235,7 +1384,7 @@ compute_row_start (GList         *siblings,
                                         &natural_width, &natural_height);
 
 
-      if (priv->vertical)
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
         natural_width = natural_height;
 
       /* if the primary axis is homogenous, each additional item is the same width */
@@ -1290,13 +1439,13 @@ mx_grid_do_allocate (ClutterActor          *self,
 
   GList *iter;
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
       priv->a_wrap = box->y2 - box->y1 - padding.top - padding.bottom;
       homogenous_b = priv->homogenous_columns;
       homogenous_a = priv->homogenous_rows;
-      aalign = priv->valign;
-      balign = priv->halign;
+      aalign = MX_ALIGN_TO_FLOAT (priv->child_y_align);
+      balign = MX_ALIGN_TO_FLOAT (priv->child_x_align);
       agap          = priv->row_spacing;
       bgap          = priv->column_spacing;
     }
@@ -1305,8 +1454,8 @@ mx_grid_do_allocate (ClutterActor          *self,
       priv->a_wrap = box->x2 - box->x1 - padding.left - padding.right;
       homogenous_a = priv->homogenous_columns;
       homogenous_b = priv->homogenous_rows;
-      aalign = priv->halign;
-      balign = priv->valign;
+      aalign = MX_ALIGN_TO_FLOAT (priv->child_x_align);
+      balign = MX_ALIGN_TO_FLOAT (priv->child_y_align);
       agap          = priv->column_spacing;
       bgap          = priv->row_spacing;
     }
@@ -1325,7 +1474,7 @@ mx_grid_do_allocate (ClutterActor          *self,
           gfloat natural_width;
           gfloat natural_height;
 
-          if (!priv->allocate_hidden && !CLUTTER_ACTOR_IS_VISIBLE (child))
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child))
             continue;
 
           /* each child will get as much space as they require */
@@ -1339,7 +1488,7 @@ mx_grid_do_allocate (ClutterActor          *self,
         }
     }
 
-  if (priv->vertical)
+  if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
       gfloat temp = priv->max_extent_a;
       priv->max_extent_a = priv->max_extent_b;
@@ -1353,7 +1502,7 @@ mx_grid_do_allocate (ClutterActor          *self,
       gfloat natural_a;
       gfloat natural_b;
 
-      if (!priv->allocate_hidden && !CLUTTER_ACTOR_IS_VISIBLE (child))
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         continue;
 
       /* each child will get as much space as they require */
@@ -1361,7 +1510,8 @@ mx_grid_do_allocate (ClutterActor          *self,
                                         NULL, NULL,
                                         &natural_a, &natural_b);
 
-      if (priv->vertical) /* swap axes around if column is major */
+      /* swap axes around if column is major */
+      if (priv->orientation == MX_ORIENTATION_VERTICAL)
         {
           gfloat temp = natural_a;
           natural_a = natural_b;
@@ -1382,7 +1532,7 @@ mx_grid_do_allocate (ClutterActor          *self,
           current_stride = 1;
         }
 
-      if (priv->end_align &&
+      if (priv->line_alignment &&
           priv->first_of_batch)
         {
           current_a = compute_row_start (iter, current_a, priv);
@@ -1422,7 +1572,7 @@ mx_grid_do_allocate (ClutterActor          *self,
         child_box.y2 = child_box.y1 + natural_b;
 
 
-        if (priv->vertical)
+        if (priv->orientation == MX_ORIENTATION_VERTICAL)
           {
             gfloat temp = child_box.x1;
             child_box.x1 = child_box.y1;
@@ -1475,7 +1625,7 @@ mx_grid_allocate (ClutterActor          *self,
   /* chain up here to preserve the allocated size
    *
    * (we ignore the height of the allocation if we have a vadjustment set,
-   *  or the width of the allocation if an vertical is set and an
+   *  or the width of the allocation if a vertical orientation is set and an
    *  hadjustment is set)
    */
   CLUTTER_ACTOR_CLASS (mx_grid_parent_class)
@@ -1483,7 +1633,7 @@ mx_grid_allocate (ClutterActor          *self,
 
 
   /* only update vadjustment - we don't really want horizontal scrolling */
-  if (priv->vadjustment && !priv->vertical)
+  if (priv->vadjustment && priv->orientation == MX_ORIENTATION_HORIZONTAL)
     {
       gdouble prev_value;
       gfloat height;
@@ -1519,7 +1669,7 @@ mx_grid_allocate (ClutterActor          *self,
       prev_value = mx_adjustment_get_value (priv->vadjustment);
       mx_adjustment_set_value (priv->vadjustment, prev_value);
     }
-  if (priv->hadjustment && priv->vertical)
+  if (priv->hadjustment && priv->orientation == MX_ORIENTATION_VERTICAL)
     {
       gdouble prev_value;
       gfloat width;
@@ -1563,11 +1713,6 @@ mx_grid_allocate (ClutterActor          *self,
                        FALSE,
                        NULL,
                        NULL);
-
-  if (priv->allocate_hidden)
-    {
-      ensure_children_are_visible ((MxGrid *) self);
-    }
 }
 
 
