@@ -758,10 +758,14 @@ handle_capture_event_cb (ClutterActor *trough,
                                                ((ClutterButtonEvent*) event)->y);
       if (target != bar->priv->handle)
         {
-          mx_stylable_set_style_pseudo_class (MX_STYLABLE (bar->priv->handle), NULL);
+          mx_stylable_set_style_pseudo_class (MX_STYLABLE (bar->priv->handle),
+                                              NULL);
+        }
+      else
+        {
+          /* allow the release event to continue to the handle for processing */
+          return FALSE;
         }
-
-
     }
 
   return TRUE;
@@ -798,7 +802,7 @@ handle_button_press_event_cb (ClutterActor       *actor,
     bar);
   g_signal_emit (bar, signals[SCROLL_START], 0);
 
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -1012,7 +1016,7 @@ stepper_button_press_event_cb (ClutterActor       *actor,
                                            stepper_button_repeat_timeout,
                                            priv);
 
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -1049,8 +1053,8 @@ mx_scroll_bar_init (MxScrollBar *self)
   self->priv = MX_SCROLL_BAR_GET_PRIVATE (self);
 
   self->priv->bw_stepper = (ClutterActor *) mx_button_new ();
-  clutter_actor_set_name (CLUTTER_ACTOR (self->priv->bw_stepper),
-                          "backward-stepper");
+  mx_stylable_set_style_class (MX_STYLABLE (self->priv->bw_stepper),
+                               "backward-stepper");
   clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->bw_stepper),
                             CLUTTER_ACTOR (self));
   g_signal_connect (self->priv->bw_stepper, "button-press-event",
@@ -1061,8 +1065,8 @@ mx_scroll_bar_init (MxScrollBar *self)
                     G_CALLBACK (stepper_button_release_cb), self);
 
   self->priv->fw_stepper = (ClutterActor *) mx_button_new ();
-  clutter_actor_set_name (CLUTTER_ACTOR (self->priv->fw_stepper),
-                          "forward-stepper");
+  mx_stylable_set_style_class (MX_STYLABLE (self->priv->fw_stepper),
+                               "forward-stepper");
   clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->fw_stepper),
                             CLUTTER_ACTOR (self));
   g_signal_connect (self->priv->fw_stepper, "button-press-event",
@@ -1074,7 +1078,7 @@ mx_scroll_bar_init (MxScrollBar *self)
 
   self->priv->trough = mx_frame_new ();
   clutter_actor_set_reactive ((ClutterActor *) self->priv->trough, TRUE);
-  clutter_actor_set_name (CLUTTER_ACTOR (self->priv->trough), "trough");
+  mx_stylable_set_style_class (MX_STYLABLE (self->priv->trough), "htrough");
   clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->trough),
                             CLUTTER_ACTOR (self));
   g_signal_connect (self->priv->trough, "button-press-event",
@@ -1085,7 +1089,7 @@ mx_scroll_bar_init (MxScrollBar *self)
                     G_CALLBACK (trough_leave_event_cb), self);
 
   self->priv->handle = (ClutterActor *) mx_button_new ();
-  clutter_actor_set_name (CLUTTER_ACTOR (self->priv->handle), "hhandle");
+  mx_stylable_set_style_class (MX_STYLABLE (self->priv->handle), "hhandle");
   clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->handle),
                             self->priv->trough);
   g_signal_connect (self->priv->handle, "button-press-event",
@@ -1207,21 +1211,25 @@ mx_scroll_bar_set_orientation (MxScrollBar   *bar,
 
       if (bar->priv->orientation)
         {
-          clutter_actor_set_name (CLUTTER_ACTOR (priv->bw_stepper),
-                                  "up-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (priv->fw_stepper),
-                                  "down-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (priv->handle),
-                                  "vhandle");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->bw_stepper),
+                                       "up-stepper");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->fw_stepper),
+                                       "down-stepper");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->handle),
+                                       "vhandle");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->trough),
+                                       "vtrough");
         }
       else
         {
-          clutter_actor_set_name (CLUTTER_ACTOR (priv->fw_stepper),
-                                  "forward-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (priv->bw_stepper),
-                                  "backward-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (priv->handle),
-                                  "hhandle");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->fw_stepper),
+                                       "forward-stepper");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->bw_stepper),
+                                       "backward-stepper");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->handle),
+                                       "hhandle");
+          mx_stylable_set_style_class (MX_STYLABLE (priv->trough),
+                                       "htrough");
         }
 
       clutter_actor_queue_relayout (CLUTTER_ACTOR (bar));
