@@ -1,8 +1,9 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-scroll-view.h: Container with scroll-bars
  *
  * Copyright 2008 OpenedHand
- * Copyright 2009 Intel Corporation.
+ * Copyright 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -47,6 +48,8 @@
 #include "mx-scroll-bar.h"
 #include "mx-scrollable.h"
 #include "mx-stylable.h"
+#include "mx-enum-types.h"
+#include "mx-private.h"
 #include <clutter/clutter.h>
 
 #include "config.h"
@@ -80,14 +83,14 @@ struct _MxScrollViewPrivate
   ClutterActor *hscroll;
   ClutterActor *vscroll;
 
-  gfloat        row_size;
-  gfloat        column_size;
-
-  gboolean      row_size_set : 1;
-  gboolean      column_size_set : 1;
   guint         mouse_scroll : 1;
   guint         enable_gestures : 1;
 
+  guint         scrollbar_width;
+  guint         scrollbar_height;
+
+  MxScrollPolicy scroll_policy;
+
 #ifdef HAVE_CLUTTER_GESTURE
   ClutterGesture *gesture;
   ClutterAnimation *animation;
@@ -97,10 +100,9 @@ struct _MxScrollViewPrivate
 enum {
   PROP_0,
 
-  PROP_HSCROLL,
-  PROP_VSCROLL,
   PROP_MOUSE_SCROLL,
-  PROP_ENABLE_GESTURES
+  PROP_ENABLE_GESTURES,
+  PROP_SCROLL_POLICY
 };
 
 static void
@@ -113,18 +115,18 @@ mx_scroll_view_get_property (GObject    *object,
 
   switch (property_id)
     {
-    case PROP_HSCROLL:
-      g_value_set_object (value, priv->hscroll);
-      break;
-    case PROP_VSCROLL:
-      g_value_set_object (value, priv->vscroll);
-      break;
     case PROP_MOUSE_SCROLL:
       g_value_set_boolean (value, priv->mouse_scroll);
       break;
+
     case PROP_ENABLE_GESTURES:
       g_value_set_boolean (value, priv->enable_gestures);
       break;
+
+    case PROP_SCROLL_POLICY:
+      g_value_set_enum (value, priv->scroll_policy);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -141,13 +143,18 @@ mx_scroll_view_set_property (GObject      *object,
   switch (property_id)
     {
     case PROP_MOUSE_SCROLL:
-      mx_scroll_view_set_mouse_scrolling (view, g_value_get_boolean (value));
+      mx_scroll_view_set_enable_mouse_scrolling (view,
+                                                 g_value_get_boolean (value));
       break;
 
     case PROP_ENABLE_GESTURES:
       mx_scroll_view_set_enable_gestures (view, g_value_get_boolean (value));
       break;
 
+    case PROP_SCROLL_POLICY:
+      mx_scroll_view_set_scroll_policy (view, g_value_get_enum (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -232,7 +239,8 @@ mx_scroll_view_get_preferred_width (ClutterActor *actor,
                                     gfloat       *natural_width_p)
 {
   MxPadding padding;
-  guint xthickness;
+  gfloat child_min_w, child_nat_w;
+  gfloat vscroll_w;
 
   MxScrollViewPrivate *priv = MX_SCROLL_VIEW (actor)->priv;
 
@@ -240,18 +248,16 @@ mx_scroll_view_get_preferred_width (ClutterActor *actor,
     return;
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
-  mx_stylable_get (MX_STYLABLE (actor),
-                   "scrollbar-width", &xthickness,
-                   NULL);
 
   /* Our natural width is the natural width of the child */
   clutter_actor_get_preferred_width (priv->child,
                                      for_height,
-                                     NULL,
-                                     natural_width_p);
+                                     &child_min_w,
+                                     &child_nat_w);
 
   /* Add space for the scroll-bar if we can determine it will be necessary */
-  if ((for_height >= 0) && natural_width_p)
+  vscroll_w = 0;
+  if (for_height >= 0)
     {
       gfloat natural_height;
 
@@ -259,15 +265,26 @@ mx_scroll_view_get_preferred_width (ClutterActor *actor,
                                           NULL,
                                           &natural_height);
       if (for_height < natural_height)
-        *natural_width_p += xthickness;
+        vscroll_w = priv->scrollbar_width;
+    }
+
+  if (min_width_p)
+    {
+      *min_width_p = padding.left + padding.right + vscroll_w;
+
+      /* if the scroll policy is not set to always or horizontal, then the
+       * minimum size of the scroll view is the minimum size of the child */
+
+      if (!(priv->scroll_policy == MX_SCROLL_POLICY_BOTH
+            || priv->scroll_policy == MX_SCROLL_POLICY_HORIZONTAL))
+        {
+          *min_width_p += child_min_w;
+        }
     }
 
   /* Add space for padding */
-  if (min_width_p)
-    *min_width_p = padding.left + padding.right;
-
   if (natural_width_p)
-    *natural_width_p += padding.left + padding.right;
+    *natural_width_p = padding.left + padding.right + child_nat_w + vscroll_w;
 }
 
 static void
@@ -277,7 +294,8 @@ mx_scroll_view_get_preferred_height (ClutterActor *actor,
                                      gfloat       *natural_height_p)
 {
   MxPadding padding;
-  guint ythickness;
+  gfloat min_child_h, nat_child_h;
+  gfloat scroll_h;
 
   MxScrollViewPrivate *priv = MX_SCROLL_VIEW (actor)->priv;
 
@@ -285,18 +303,16 @@ mx_scroll_view_get_preferred_height (ClutterActor *actor,
     return;
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
-  mx_stylable_get (MX_STYLABLE (actor),
-                   "scrollbar-height", &ythickness,
-                   NULL);
 
   /* Our natural height is the natural height of the child */
   clutter_actor_get_preferred_height (priv->child,
                                       for_width,
-                                      NULL,
-                                      natural_height_p);
+                                      &min_child_h,
+                                      &nat_child_h);
 
   /* Add space for the scroll-bar if we can determine it will be necessary */
-  if ((for_width >= 0) && natural_height_p)
+  scroll_h = 0;
+  if (for_width >= 0)
     {
       gfloat natural_width;
 
@@ -304,15 +320,23 @@ mx_scroll_view_get_preferred_height (ClutterActor *actor,
                                          NULL,
                                          &natural_width);
       if (for_width < natural_width)
-        *natural_height_p += ythickness;
+        scroll_h = priv->scrollbar_height;
     }
 
   /* Add space for padding */
   if (min_height_p)
-    *min_height_p = padding.top + padding.bottom;
+    {
+      *min_height_p = padding.top + padding.bottom + scroll_h;
+
+      if (!(priv->scroll_policy == MX_SCROLL_POLICY_BOTH
+            || priv->scroll_policy == MX_SCROLL_POLICY_VERTICAL))
+        {
+          *min_height_p += min_child_h;
+        }
+    }
 
   if (natural_height_p)
-    *natural_height_p += padding.top + padding.bottom;
+    *natural_height_p = padding.top + nat_child_h + padding.bottom + scroll_h;
 }
 
 static void
@@ -345,12 +369,8 @@ mx_scroll_view_allocate (ClutterActor          *actor,
   avail_width = (box->x2 - box->x1) - padding.left - padding.right;
   avail_height = (box->y2 - box->y1) - padding.top - padding.bottom;
 
-  mx_stylable_get (MX_STYLABLE (actor),
-                   "scrollbar-width", &sb_width,
-                   "scrollbar-height", &sb_height,
-                   NULL);
-  sb_width = 28;
-  sb_height = 28;
+  sb_width = priv->scrollbar_width;
+  sb_height = priv->scrollbar_height;
 
   if (!CLUTTER_ACTOR_IS_VISIBLE (priv->vscroll))
     sb_width = 0;
@@ -392,12 +412,17 @@ mx_scroll_view_allocate (ClutterActor          *actor,
 }
 
 static void
-mx_scroll_view_style_changed (MxWidget *widget)
+mx_scroll_view_style_changed (MxWidget *widget, MxStyleChangedFlags flags)
 {
   MxScrollViewPrivate *priv = MX_SCROLL_VIEW (widget)->priv;
 
-  mx_stylable_changed ((MxStylable *) priv->hscroll);
-  mx_stylable_changed ((MxStylable *) priv->vscroll);
+  mx_stylable_style_changed (MX_STYLABLE (priv->hscroll), flags);
+  mx_stylable_style_changed (MX_STYLABLE (priv->vscroll), flags);
+
+  mx_stylable_get (MX_STYLABLE (widget),
+                   "x-mx-scrollbar-width", &priv->scrollbar_width,
+                   "x-mx-scrollbar-height", &priv->scrollbar_height,
+                   NULL);
 }
 
 static gboolean
@@ -431,7 +456,7 @@ mx_scroll_view_scroll_event (ClutterActor       *self,
       break;
     case CLUTTER_SCROLL_LEFT:
     case CLUTTER_SCROLL_RIGHT:
-      if (vadjustment)
+      if (hadjustment)
         g_object_get (hadjustment,
                       "lower", &lower,
                       "step-increment", &step,
@@ -449,25 +474,29 @@ mx_scroll_view_scroll_event (ClutterActor       *self,
       if (value == lower)
         return FALSE;
       else
-        mx_adjustment_set_value (vadjustment, value - step);
+        mx_adjustment_interpolate_relative (vadjustment, -step,
+                                            250, CLUTTER_EASE_OUT_CUBIC);
       break;
     case CLUTTER_SCROLL_DOWN:
       if (value == upper)
         return FALSE;
       else
-        mx_adjustment_set_value (vadjustment, value + step);
+        mx_adjustment_interpolate_relative (vadjustment, step,
+                                            250, CLUTTER_EASE_OUT_CUBIC);
       break;
     case CLUTTER_SCROLL_LEFT:
       if (value == lower)
         return FALSE;
       else
-        mx_adjustment_set_value (hadjustment, value - step);
+        mx_adjustment_interpolate_relative (hadjustment, -step,
+                                            250, CLUTTER_EASE_OUT_CUBIC);
       break;
     case CLUTTER_SCROLL_RIGHT:
       if (value == upper)
         return FALSE;
       else
-        mx_adjustment_set_value (hadjustment, value + step);
+        mx_adjustment_interpolate_relative (hadjustment, step,
+                                            250, CLUTTER_EASE_OUT_CUBIC);
       break;
     }
 
@@ -495,27 +524,11 @@ mx_scroll_view_class_init (MxScrollViewClass *klass)
   actor_class->allocate = mx_scroll_view_allocate;
   actor_class->scroll_event = mx_scroll_view_scroll_event;
 
-  g_object_class_install_property (object_class,
-                                   PROP_HSCROLL,
-                                   g_param_spec_object ("hscroll",
-                                                        "MxScrollBar",
-                                                        "Horizontal scroll indicator",
-                                                        MX_TYPE_SCROLL_BAR,
-                                                        G_PARAM_READABLE));
-
-  g_object_class_install_property (object_class,
-                                   PROP_VSCROLL,
-                                   g_param_spec_object ("vscroll",
-                                                        "MxScrollBar",
-                                                        "Vertical scroll indicator",
-                                                        MX_TYPE_SCROLL_BAR,
-                                                        G_PARAM_READABLE));
-
   pspec = g_param_spec_boolean ("enable-mouse-scrolling",
                                 "Enable Mouse Scrolling",
                                 "Enable automatic mouse wheel scrolling",
                                 TRUE,
-                                G_PARAM_READWRITE);
+                                MX_PARAM_READWRITE);
   g_object_class_install_property (object_class,
                                    PROP_MOUSE_SCROLL,
                                    pspec);
@@ -524,8 +537,16 @@ mx_scroll_view_class_init (MxScrollViewClass *klass)
                                 "Enable Gestures",
                                 "Enable use of pointer gestures for scrolling",
                                 FALSE,
-                                G_PARAM_READWRITE);
+                                MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_ENABLE_GESTURES, pspec);
+
+  pspec = g_param_spec_enum ("scroll-policy",
+                             "Scroll Policy",
+                             "The scroll policy",
+                             MX_TYPE_SCROLL_POLICY,
+                             MX_SCROLL_POLICY_BOTH,
+                             MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_SCROLL_POLICY, pspec);
 }
 
 static void
@@ -539,14 +560,14 @@ mx_stylable_iface_init (MxStylableIface *iface)
 
       is_initialized = TRUE;
 
-      pspec = g_param_spec_uint ("scrollbar-width",
+      pspec = g_param_spec_uint ("x-mx-scrollbar-width",
                                  "Vertical scroll-bar thickness",
                                  "Thickness of vertical scrollbar, in px",
                                  0, G_MAXUINT, 24,
                                  G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface, MX_TYPE_SCROLL_VIEW, pspec);
 
-      pspec = g_param_spec_uint ("scrollbar-height",
+      pspec = g_param_spec_uint ("x-mx-scrollbar-height",
                                  "Horizontal scroll-bar thickness",
                                  "Thickness of horizontal scrollbar, in px",
                                  0, G_MAXUINT, 24,
@@ -598,14 +619,6 @@ child_hadjustment_notify_cb (GObject    *gobject,
   mx_scrollable_get_adjustments (MX_SCROLLABLE (actor), &hadjust, NULL);
   if (hadjust)
     {
-      /* Force scroll step if neede. */
-      if (priv->column_size_set)
-        {
-          g_object_set (hadjust,
-                        "step-increment", priv->column_size,
-                        NULL);
-        }
-
       mx_scroll_bar_set_adjustment (MX_SCROLL_BAR(priv->hscroll), hadjust);
       g_signal_connect (hadjust, "changed", G_CALLBACK (
                           child_adjustment_changed_cb), priv->hscroll);
@@ -632,14 +645,6 @@ child_vadjustment_notify_cb (GObject    *gobject,
   mx_scrollable_get_adjustments (MX_SCROLLABLE(actor), NULL, &vadjust);
   if (vadjust)
     {
-      /* Force scroll step if neede. */
-      if (priv->row_size_set)
-        {
-          g_object_set (vadjust,
-                        "step-increment", priv->row_size,
-                        NULL);
-        }
-
       mx_scroll_bar_set_adjustment (MX_SCROLL_BAR(priv->vscroll), vadjust);
       g_signal_connect (vadjust, "changed", G_CALLBACK (
                           child_adjustment_changed_cb), priv->vscroll);
@@ -728,11 +733,17 @@ mx_scroll_view_init (MxScrollView *self)
 {
   MxScrollViewPrivate *priv = self->priv = SCROLL_VIEW_PRIVATE (self);
 
-  priv->hscroll = CLUTTER_ACTOR (mx_scroll_bar_new (NULL));
-  priv->vscroll = g_object_new (MX_TYPE_SCROLL_BAR, "vertical", TRUE, NULL);
+  priv->hscroll = mx_scroll_bar_new ();
+  priv->vscroll = g_object_new (MX_TYPE_SCROLL_BAR,
+                                "orientation", MX_ORIENTATION_VERTICAL,
+                                NULL);
+
+  priv->scroll_policy = MX_SCROLL_POLICY_BOTH;
 
   clutter_actor_set_parent (priv->hscroll, CLUTTER_ACTOR (self));
   clutter_actor_set_parent (priv->vscroll, CLUTTER_ACTOR (self));
+  clutter_actor_hide (priv->hscroll);
+  clutter_actor_hide (priv->vscroll);
 
   /* mouse scroll is enabled by default, so we also need to be reactive */
   priv->mouse_scroll = TRUE;
@@ -742,14 +753,6 @@ mx_scroll_view_init (MxScrollView *self)
   g_signal_connect (self, "style-changed",
                     G_CALLBACK (mx_scroll_view_style_changed), NULL);
 
-#ifdef HAVE_CLUTTER_GESTURE
-  priv->gesture = clutter_gesture_new (CLUTTER_ACTOR (self));
-  clutter_gesture_set_gesture_mask (priv->gesture, CLUTTER_ACTOR (self),
-                                    GESTURE_MASK_SLIDE);
-  g_signal_connect (priv->gesture, "gesture-slide-event",
-                    G_CALLBACK (mx_scroll_view_gesture_slide_event_cb),
-                    self);
-#endif
 }
 
 static void
@@ -767,10 +770,10 @@ mx_scroll_view_add (ClutterContainer *container,
       mx_scroll_view_parent_iface->add (container, actor);
 
       /* Get adjustments for scroll-bars */
-      g_signal_connect (actor, "notify::hadjustment",
+      g_signal_connect (actor, "notify::horizontal-adjustment",
                         G_CALLBACK (child_hadjustment_notify_cb),
                         container);
-      g_signal_connect (actor, "notify::vadjustment",
+      g_signal_connect (actor, "notify::vertical-adjustment",
                         G_CALLBACK (child_vadjustment_notify_cb),
                         container);
       child_hadjustment_notify_cb (G_OBJECT (actor), NULL, container);
@@ -842,121 +845,23 @@ clutter_container_iface_init (ClutterContainerIface *iface)
   iface->foreach_with_internals = mx_scroll_view_foreach_with_internals;
 }
 
+/**
+ * mx_scroll_view_get_hscroll_bar:
+ * @scroll: a #MxScrollView
+ *
+ * Gets the horizontal scrollbar of the scrollbiew
+ *
+ * Return value: (transfer none): the horizontal #MxScrollbar
+ */
 ClutterActor *
 mx_scroll_view_new (void)
 {
   return g_object_new (MX_TYPE_SCROLL_VIEW, NULL);
 }
 
-ClutterActor *
-mx_scroll_view_get_hscroll_bar (MxScrollView *scroll)
-{
-  g_return_val_if_fail (MX_IS_SCROLL_VIEW (scroll), NULL);
-
-  return scroll->priv->hscroll;
-}
-
-ClutterActor *
-mx_scroll_view_get_vscroll_bar (MxScrollView *scroll)
-{
-  g_return_val_if_fail (MX_IS_SCROLL_VIEW (scroll), NULL);
-
-  return scroll->priv->vscroll;
-}
-
-gfloat
-mx_scroll_view_get_column_size (MxScrollView *scroll)
-{
-  MxAdjustment *adjustment;
-  gdouble column_size;
-
-  g_return_val_if_fail (scroll, 0);
-
-  adjustment = mx_scroll_bar_get_adjustment (
-    MX_SCROLL_BAR (scroll->priv->hscroll));
-  g_object_get (adjustment,
-                "step-increment", &column_size,
-                NULL);
-
-  return column_size;
-}
-
 void
-mx_scroll_view_set_column_size (MxScrollView *scroll,
-                                gfloat        column_size)
-{
-  MxAdjustment *adjustment;
-
-  g_return_if_fail (scroll);
-
-  if (column_size < 0)
-    {
-      scroll->priv->column_size_set = FALSE;
-      scroll->priv->column_size = -1;
-    }
-  else
-    {
-      scroll->priv->column_size_set = TRUE;
-      scroll->priv->column_size = column_size;
-
-      adjustment = mx_scroll_bar_get_adjustment (
-        MX_SCROLL_BAR (scroll->priv->hscroll));
-
-      if (adjustment)
-        g_object_set (adjustment,
-                      "step-increment", (gdouble) scroll->priv->column_size,
-                      NULL);
-    }
-}
-
-gfloat
-mx_scroll_view_get_row_size (MxScrollView *scroll)
-{
-  MxAdjustment *adjustment;
-  gdouble row_size;
-
-  g_return_val_if_fail (scroll, 0);
-
-  adjustment = mx_scroll_bar_get_adjustment (
-    MX_SCROLL_BAR (scroll->priv->vscroll));
-  g_object_get (adjustment,
-                "step-increment", &row_size,
-                NULL);
-
-  return row_size;
-}
-
-void
-mx_scroll_view_set_row_size (MxScrollView *scroll,
-                             gfloat        row_size)
-{
-  MxAdjustment *adjustment;
-
-  g_return_if_fail (scroll);
-
-  if (row_size < 0)
-    {
-      scroll->priv->row_size_set = FALSE;
-      scroll->priv->row_size = -1;
-    }
-  else
-    {
-      scroll->priv->row_size_set = TRUE;
-      scroll->priv->row_size = row_size;
-
-      adjustment = mx_scroll_bar_get_adjustment (
-        MX_SCROLL_BAR (scroll->priv->vscroll));
-
-      if (adjustment)
-        g_object_set (adjustment,
-                      "step-increment", (gdouble) scroll->priv->row_size,
-                      NULL);
-    }
-}
-
-void
-mx_scroll_view_set_mouse_scrolling (MxScrollView *scroll,
-                                    gboolean      enabled)
+mx_scroll_view_set_enable_mouse_scrolling (MxScrollView *scroll,
+                                           gboolean      enabled)
 {
   MxScrollViewPrivate *priv;
 
@@ -971,11 +876,13 @@ mx_scroll_view_set_mouse_scrolling (MxScrollView *scroll,
       /* make sure we can receive mouse wheel events */
       if (enabled)
         clutter_actor_set_reactive ((ClutterActor *) scroll, TRUE);
+
+      g_object_notify (G_OBJECT (scroll), "enable-mouse-scrolling");
     }
 }
 
 gboolean
-mx_scroll_view_get_mouse_scrolling (MxScrollView *scroll)
+mx_scroll_view_get_enable_mouse_scrolling (MxScrollView *scroll)
 {
   MxScrollViewPrivate *priv;
 
@@ -1002,6 +909,18 @@ mx_scroll_view_set_enable_gestures (MxScrollView *scroll,
 
 #ifndef HAVE_CLUTTER_GESTURE
       g_warning ("Gestures are disabled as Clutter Gesture is not available");
+#else
+      if (enabled && !priv->gesture)
+        {
+          priv->gesture = clutter_gesture_new (CLUTTER_ACTOR (scroll));
+          clutter_gesture_set_gesture_mask (priv->gesture,
+                                            CLUTTER_ACTOR (scroll),
+                                            GESTURE_MASK_SLIDE);
+          g_signal_connect (priv->gesture, "gesture-slide-event",
+                            G_CALLBACK (mx_scroll_view_gesture_slide_event_cb),
+                            scroll);
+          clutter_actor_set_reactive (CLUTTER_ACTOR (scroll), TRUE);
+        }
 #endif
 
       g_object_notify (G_OBJECT (scroll), "enable-gestures");
@@ -1015,3 +934,100 @@ mx_scroll_view_get_enable_gestures (MxScrollView *scroll)
 
   return scroll->priv->enable_gestures;
 }
+
+
+void
+mx_scroll_view_set_scroll_policy (MxScrollView  *scroll,
+                                  MxScrollPolicy policy)
+{
+  MxScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_SCROLL_VIEW (scroll));
+
+  priv = scroll->priv;
+
+  if (priv->scroll_policy != policy)
+    {
+      priv->scroll_policy = policy;
+
+      g_object_notify (G_OBJECT (scroll), "scroll-policy");
+
+      clutter_actor_queue_relayout (CLUTTER_ACTOR (scroll));
+    }
+}
+
+MxScrollPolicy
+mx_scroll_view_get_scroll_policy (MxScrollView *scroll)
+{
+  g_return_val_if_fail (MX_IS_SCROLL_VIEW (scroll), 0);
+
+  return scroll->priv->scroll_policy;
+}
+
+static void
+_mx_scroll_view_ensure_visible_axis (MxScrollBar  *bar,
+                                     gdouble       lower,
+                                     gdouble       upper)
+{
+  gdouble new_value, adjust_lower, adjust_upper, adjust_page_size;
+
+  gboolean changed = FALSE;
+  MxAdjustment *adjust = mx_scroll_bar_get_adjustment (bar);
+
+  mx_adjustment_get_values (adjust,
+                            &new_value,
+                            &adjust_lower,
+                            &adjust_upper,
+                            NULL,
+                            NULL,
+                            &adjust_page_size);
+
+  /* Sanitise input values */
+  lower = CLAMP (lower, adjust_lower, adjust_upper - adjust_page_size);
+  upper = CLAMP (upper, adjust_lower + adjust_page_size, adjust_upper);
+
+  /* Ensure the bottom is visible */
+  if (new_value + adjust_page_size < upper)
+    {
+      new_value = upper - adjust_page_size;
+      changed = TRUE;
+    }
+
+  /* Ensure the top is visible */
+  if (lower < new_value)
+    {
+      new_value = lower;
+      changed = TRUE;
+    }
+
+  if (changed)
+    mx_adjustment_interpolate (adjust, new_value,
+                               250, CLUTTER_EASE_OUT_CUBIC);
+}
+
+/**
+ * mx_scroll_view_ensure_visible:
+ * @scroll: A #MxScrollView
+ * @geometry: The region to make visible
+ *
+ * Ensures that a given region is visible in the ScrollView, with the top-left
+ * taking precedence.
+ *
+ */
+void
+mx_scroll_view_ensure_visible (MxScrollView          *scroll,
+                               const ClutterGeometry *geometry)
+{
+  MxScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_SCROLL_VIEW (scroll));
+
+  priv = scroll->priv;
+
+  _mx_scroll_view_ensure_visible_axis (MX_SCROLL_BAR (priv->hscroll),
+                                       geometry->x,
+                                       geometry->x + geometry->width);
+  _mx_scroll_view_ensure_visible_axis (MX_SCROLL_BAR (priv->vscroll),
+                                       geometry->y,
+                                       geometry->y + geometry->height);
+}
