@@ -66,16 +66,18 @@ struct _MxSliderPrivate
   /* keep those around for ::alocate_fill() */
   gfloat        trough_box_y1;
   gfloat        trough_box_y2;
-  gint          handle_width;
+  gint          trough_height;
+  guint         handle_width;
+  guint         handle_height;
 
-  gdouble       progress;
+  gdouble       value;
 };
 
 enum
 {
   PROP_0,
 
-  PROP_PROGRESS
+  PROP_VALUE
 };
 
 static void
@@ -84,7 +86,7 @@ drag_handle (MxSlider *bar,
              gfloat    y)
 {
   MxSliderPrivate *priv = bar->priv;
-  gdouble progress;
+  gdouble value;
   gfloat ux, pos, handle_width_2, fill_size, offset;
 
   if (!clutter_actor_transform_stage_point (CLUTTER_ACTOR (bar),
@@ -104,8 +106,8 @@ drag_handle (MxSlider *bar,
   pos = ux - priv->handle_middle_start + offset;
   pos = CLAMP (pos, 0, fill_size);
 
-  progress = pos / fill_size;
-  mx_slider_set_progress (bar, progress);
+  value = pos / fill_size;
+  mx_slider_set_value (bar, value);
 }
 
 static gboolean
@@ -159,7 +161,7 @@ move_handle (MxSlider *bar,
              gfloat    y)
 {
   MxSliderPrivate *priv = bar->priv;
-  gdouble progress;
+  gdouble value;
   gfloat ux, pos, fill_size;
 
   if (!clutter_actor_transform_stage_point (CLUTTER_ACTOR (bar),
@@ -175,8 +177,8 @@ move_handle (MxSlider *bar,
   pos = ux - priv->handle_middle_start;
   pos = CLAMP (pos, 0, fill_size);
 
-  progress = pos / fill_size;
-  mx_slider_set_progress (bar, progress);
+  value = pos / fill_size;
+  mx_slider_set_value (bar, value);
 }
 
 static gboolean
@@ -189,6 +191,10 @@ on_trough_bg_button_press_event (ClutterActor       *actor,
   if (event->button != 1)
     return FALSE;
 
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return FALSE;
+
+
   move_handle (self, event->x, event->y);
 
   /* Turn off picking for motion events */
@@ -229,6 +235,9 @@ on_handle_button_press_event (ClutterActor       *actor,
 {
   MxSliderPrivate *priv = bar->priv;
 
+  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+    return FALSE;
+
   if (event->button != 1)
     return FALSE;
 
@@ -271,17 +280,17 @@ mx_stylable_iface_init (MxStylableIface *iface)
 
       /* if specified, this will be the allocated height of the trough.
        * By default, the height of the trough is the same as its parent. */
-      pspec = g_param_spec_int ("trough-height",
+      pspec = g_param_spec_int ("x-mx-trough-height",
                                 "Height of the trough",
                                 "Height of the trough, in px",
-                                -1, G_MAXUINT, -1,
+                                -1, G_MAXINT, -1,
                                 G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface,
                                           MX_TYPE_SLIDER, pspec);
 
       /* FIXME: have a trough-width property too? */
 
-      pspec = g_param_spec_uint ("handle-width",
+      pspec = g_param_spec_uint ("x-mx-handle-width",
                                  "Handle width",
                                  "Width of the handle, in px",
                                  0, G_MAXUINT, DEFAULT_HANDLE_WIDTH,
@@ -289,7 +298,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
       mx_stylable_iface_install_property (iface,
                                           MX_TYPE_SLIDER, pspec);
 
-      pspec = g_param_spec_uint ("handle-height",
+      pspec = g_param_spec_uint ("x-mx-handle-height",
                                  "Handle height",
                                  "Height of the handle, in px",
                                  0, G_MAXUINT, DEFAULT_HANDLE_HEIGHT,
@@ -313,7 +322,7 @@ mx_slider_paint (ClutterActor *actor)
 
   clutter_actor_paint (priv->trough_bg);
 
-  if (priv->progress)
+  if (priv->value)
     clutter_actor_paint (priv->fill);
 
   clutter_actor_paint (priv->trough);
@@ -336,6 +345,48 @@ mx_slider_pick (ClutterActor       *actor,
   clutter_actor_paint (priv->handle);
 }
 
+static void
+mx_slider_get_preferred_width (ClutterActor *actor,
+                               gfloat        for_height,
+                               gfloat       *min_width_p,
+                               gfloat       *nat_width_p)
+{
+  MxPadding padding;
+  MxSliderPrivate *priv = MX_SLIDER (actor)->priv;
+
+  mx_widget_get_padding (MX_WIDGET (actor), &padding);
+
+  /* Set the size of the handle + padding */
+  if (min_width_p)
+    *min_width_p = priv->handle_width + padding.left + padding.right;
+
+  /* Set the preferred size as some arbitrary value larger than the handle 
+   * width. This ensures the slider can actually move!
+   */
+  if (nat_width_p)
+    *nat_width_p = (priv->handle_width * 4) + padding.left + padding.right;
+}
+
+static void
+mx_slider_get_preferred_height (ClutterActor *actor,
+                                gfloat        for_width,
+                                gfloat       *min_height_p,
+                                gfloat       *nat_height_p)
+{
+  MxPadding padding;
+  MxSliderPrivate *priv = MX_SLIDER (actor)->priv;
+
+  mx_widget_get_padding (MX_WIDGET (actor), &padding);
+
+  /* Add on the size of the trough/handle + padding */
+  if (min_height_p)
+    *min_height_p = MAX (priv->handle_height, priv->trough_height) +
+                    padding.top + padding.bottom;
+  if (nat_height_p)
+    *nat_height_p = MAX (priv->handle_height, priv->trough_height) +
+                    padding.top + padding.bottom;
+}
+
 static void
 mx_slider_allocate_fill_handle (MxSlider               *self,
                                 const ClutterActorBox  *box,
@@ -362,7 +413,7 @@ mx_slider_allocate_fill_handle (MxSlider               *self,
   fill_box.x1 = padding.left;
   fill_box.y1 = priv->trough_box_y1;
   fill_box.x2 = ((box->x2 - box->x1 - padding.left - padding.right -
-                  priv->handle_width) * priv->progress) + padding.left +
+                  priv->handle_width) * priv->value) + padding.left +
                   handle_width_2;
   fill_box.x2 = CLAMP (fill_box.x2,
                        priv->handle_middle_start,
@@ -373,9 +424,22 @@ mx_slider_allocate_fill_handle (MxSlider               *self,
 
   /* handle */
   handle_box.x1 = fill_box.x2 - handle_width_2;
-  handle_box.y1 = padding.top;
   handle_box.x2 = handle_box.x1 + priv->handle_width;
-  handle_box.y2 = (box->y2 - box->y1) - padding.bottom;
+
+  /* If the handle height is unset, or the handle height is greater than the
+   * trough height, we want the handle to occupy all available space.
+   * Otherwise we want it to be centred in the trough.
+   */
+  if ((priv->handle_height == 0) || (priv->handle_height > priv->trough_height))
+    {
+      handle_box.y1 = padding.top;
+      handle_box.y2 = (box->y2 - box->y1) - padding.bottom;
+    }
+  else
+    {
+      handle_box.y1 = (box->y2 - box->y1 - priv->handle_height) / 2.f;
+      handle_box.y2 = handle_box.y1 + priv->handle_height;
+    }
 
   /* snap to pixel */
   handle_box.x1 = (int) handle_box.x1;
@@ -397,8 +461,7 @@ mx_slider_allocate (ClutterActor           *actor,
   ClutterActorClass *actor_class;
   ClutterActorBox    bar_box;
   ClutterActorBox    trough_box;
-  guint              handle_width_2, handle_height;
-  gint               trough_height;
+  guint              handle_width_2;
 
   actor_class = CLUTTER_ACTOR_CLASS (mx_slider_parent_class);
   actor_class->allocate (actor, box, flags);
@@ -409,11 +472,6 @@ mx_slider_allocate (ClutterActor           *actor,
       box = &bar_box;
     }
 
-  mx_stylable_get (MX_STYLABLE (self),
-                   "trough-height", &trough_height,
-                   "handle-width",  &priv->handle_width,
-                   "handle-height",  &handle_height,
-                   NULL);
   handle_width_2 = priv->handle_width >> 1;
 
   mx_widget_get_padding (MX_WIDGET (self), &padding);
@@ -423,7 +481,7 @@ mx_slider_allocate (ClutterActor           *actor,
   priv->handle_middle_end   = box->x2 - box->x1 - padding.right -
     handle_width_2 - 1;
 
-  if (trough_height < 0)
+  if (priv->trough_height < 0)
     {
       /* trough-height has not been specified, take the whole height */
       trough_box.x1 = padding.left;
@@ -435,9 +493,9 @@ mx_slider_allocate (ClutterActor           *actor,
     {
       trough_box.x1 = padding.left;
       trough_box.y1 = (box->y2 - box->y1 - padding.bottom - padding.top -
-                       trough_height) / 2;
+                       priv->trough_height) / 2;
       trough_box.x2 = (box->x2 - box->x1) - padding.right;
-      trough_box.y2 = trough_box.y1 + trough_height;
+      trough_box.y2 = trough_box.y1 + priv->trough_height;
     }
 
   clutter_actor_allocate (priv->trough_bg, &trough_box, flags);
@@ -492,8 +550,8 @@ mx_slider_get_property (GObject    *object,
 
   switch (property_id)
     {
-    case PROP_PROGRESS:
-      g_value_set_double (value, mx_slider_get_progress (self));
+    case PROP_VALUE:
+      g_value_set_double (value, mx_slider_get_value (self));
       break;
 
     default:
@@ -511,8 +569,8 @@ mx_slider_set_property (GObject      *object,
 
   switch (property_id)
     {
-    case PROP_PROGRESS:
-      mx_slider_set_progress (self, g_value_get_double (value));
+    case PROP_VALUE:
+      mx_slider_set_value (self, g_value_get_double (value));
       break;
 
     default:
@@ -579,15 +637,61 @@ mx_slider_class_init (MxSliderClass *klass)
 
   actor_class->paint = mx_slider_paint;
   actor_class->pick = mx_slider_pick;
+  actor_class->get_preferred_width = mx_slider_get_preferred_width;
+  actor_class->get_preferred_height = mx_slider_get_preferred_height;
   actor_class->allocate = mx_slider_allocate;
   actor_class->map = mx_slider_map;
   actor_class->unmap = mx_slider_unmap;
 
-  pspec = g_param_spec_double ("progress",
-                               "Progress",
-                               "Progress",
+  pspec = g_param_spec_double ("value",
+                               "Value",
+                               "Value",
                                0.0, 1.0, 0.0, G_PARAM_READWRITE);
-  g_object_class_install_property (object_class, PROP_PROGRESS, pspec);
+  g_object_class_install_property (object_class, PROP_VALUE, pspec);
+}
+
+static void
+mx_slider_style_changed_cb (MxSlider *self)
+{
+  gboolean relayout;
+  gint trough_height;
+  guint handle_width, handle_height;
+
+  MxSliderPrivate *priv = self->priv;
+
+  mx_stylable_get (MX_STYLABLE (self),
+                   "x-mx-trough-height", &trough_height,
+                   "x-mx-handle-width",  &handle_width,
+                   "x-mx-handle-height", &handle_height,
+                   NULL);
+
+  relayout = FALSE;
+
+  if (priv->trough_height != trough_height)
+    {
+      priv->trough_height = trough_height;
+      relayout = TRUE;
+    }
+
+  if (priv->handle_width != handle_width)
+    {
+      priv->handle_width = handle_width;
+      relayout = TRUE;
+    }
+
+  if (priv->handle_height != handle_height)
+    {
+      priv->handle_height = handle_height;
+      relayout = TRUE;
+    }
+
+  mx_stylable_style_changed (MX_STYLABLE (priv->trough_bg), 0);
+  mx_stylable_style_changed (MX_STYLABLE (priv->fill), 0);
+  mx_stylable_style_changed (MX_STYLABLE (priv->trough), 0);
+  mx_stylable_style_changed (MX_STYLABLE (priv->handle), 0);
+
+  if (relayout)
+    clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
 }
 
 static void
@@ -597,6 +701,9 @@ mx_slider_init (MxSlider *self)
 
   self->priv = priv = SLIDER_PRIVATE (self);
 
+  g_signal_connect (self, "style-changed",
+                    G_CALLBACK (mx_slider_style_changed_cb), NULL);
+
   priv->trough_bg = CLUTTER_ACTOR (_mx_progress_bar_fill_new ());
   clutter_actor_set_name (priv->trough_bg, "trough-background");
   clutter_actor_set_reactive (priv->trough_bg, TRUE);
@@ -618,7 +725,7 @@ mx_slider_init (MxSlider *self)
 
   self->priv->handle = CLUTTER_ACTOR (mx_button_new ());
   clutter_actor_set_name (priv->handle, "handle");
-  clutter_actor_set_parent (priv->handle, priv->trough);
+  clutter_actor_set_parent (priv->handle, CLUTTER_ACTOR (self));
   g_signal_connect (priv->handle, "button-press-event",
                     G_CALLBACK (on_handle_button_press_event), self);
 }
@@ -637,49 +744,49 @@ mx_slider_new (void)
 }
 
 /**
- * mx_slider_set_progress:
- * @bar: A #MxProgressBar
- * @progress: A value between 0.0 and 1.0
+ * mx_slider_set_value:
+ * @bar: A #MxSlider
+ * @value: A value between 0.0 and 1.0
  *
- * Set the progress of the slider
+ * Set the value of the slider
  */
 void
-mx_slider_set_progress (MxSlider *bar,
-                        gdouble      progress)
+mx_slider_set_value (MxSlider *bar,
+                     gdouble   value)
 {
   MxSliderPrivate *priv = bar->priv;
 
   g_return_if_fail (MX_IS_SLIDER (bar));
 
-  if (priv->progress == progress)
+  if (priv->value == value)
     return;
 
-  if (G_UNLIKELY ((progress < 0.0) || (progress > 1.0)))
+  if (G_UNLIKELY ((value < 0.0) || (value > 1.0)))
     {
-      g_warning ("progress must be a number between 0.0 and 1.0");
+      g_warning ("MxSlider:value must be a number between 0.0 and 1.0");
       return;
     }
 
-  priv->progress = progress;
+  priv->value = value;
 
   mx_slider_allocate_fill_handle (bar, NULL, 0);
   clutter_actor_queue_redraw (CLUTTER_ACTOR (bar));
 
-  g_object_notify (G_OBJECT (bar), "progress");
+  g_object_notify (G_OBJECT (bar), "value");
 }
 
 /**
- * mx_slider_get_progress:
- * @bar: A #MxProgressBar
+ * mx_slider_get_value:
+ * @bar: A #MxSlider
  *
- * Retrieve the current progress of the media bar
+ * Retrieve the current value of the media bar
  *
  * Returns: gdouble
  */
 gdouble
-mx_slider_get_progress (MxSlider *bar)
+mx_slider_get_value (MxSlider *bar)
 {
   g_return_val_if_fail (MX_IS_SLIDER (bar), 0.0);
 
-  return bar->priv->progress;
+  return bar->priv->value;
 }
