@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-stylable.c: Interface for stylable objects
  *
@@ -48,7 +49,9 @@
 enum
 {
   STYLE_CHANGED,
+#if 0
   STYLE_NOTIFY,
+#endif
   CHANGED,
 
   LAST_SIGNAL
@@ -68,12 +71,15 @@ mx_stylable_notify_dispatcher (GObject     *gobject,
                                guint        n_pspecs,
                                GParamSpec **pspecs)
 {
+#if 0
   guint i;
 
   for (i = 0; i < n_pspecs; i++)
     g_signal_emit (gobject, stylable_signals[STYLE_NOTIFY],
                    g_quark_from_string (pspecs[i]->name),
                    pspecs[i]);
+#endif
+
 }
 
 static void
@@ -152,25 +158,10 @@ mx_stylable_base_init (gpointer g_iface)
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (MxStylableIface, style_changed),
                   NULL, NULL,
-                  _mx_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
-
-  /**
-   * MxStylable::stylable-changed:
-   * @actor: the actor that received the signal
-   *
-   * The ::changed signal is emitted each time any of the properties of the
-   * stylable has changed.
-   */
-  stylable_signals[CHANGED] =
-    g_signal_new (I_("stylable-changed"),
-                  iface_type,
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (MxStylableIface, stylable_changed),
-                  NULL, NULL,
-                  _mx_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
+                  _mx_marshal_VOID__FLAGS,
+                  G_TYPE_NONE, 1, MX_TYPE_STYLE_CHANGED_FLAGS);
 
+#if 0
   stylable_signals[STYLE_NOTIFY] =
     g_signal_new (I_("style-notify"),
                   iface_type,
@@ -181,6 +172,7 @@ mx_stylable_base_init (gpointer g_iface)
                   _mx_marshal_VOID__PARAM,
                   G_TYPE_NONE, 1,
                   G_TYPE_PARAM);
+#endif
 }
 
 GType
@@ -204,6 +196,7 @@ mx_stylable_get_type (void)
   return our_type;
 }
 
+#if 0
 void
 mx_stylable_freeze_notify (MxStylable *stylable)
 {
@@ -268,6 +261,7 @@ mx_stylable_notify (MxStylable  *stylable,
 
   g_object_unref (stylable);
 }
+#endif
 
 /**
  * mx_stylable_iface_install_property:
@@ -343,12 +337,12 @@ mx_stylable_iface_install_property (MxStylableIface *iface,
 /**
  * mx_stylable_list_properties:
  * @stylable: a #MxStylable
- * @n_props: return location for the number of properties, or %NULL
+ * @n_props: (out): return location for the number of properties, or %NULL
  *
  * Retrieves all the #GParamSpec<!-- -->s installed by @stylable.
  *
- * Return value: an array of #GParamSpec<!-- -->s. Free it with
- *   g_free() when done.
+ * Return value: (transfer container) (array length=n_props): an array
+ *  of #GParamSpec<!-- -->s. Free it with  g_free() when done.
  */
 GParamSpec **
 mx_stylable_list_properties (MxStylable *stylable,
@@ -376,8 +370,8 @@ mx_stylable_list_properties (MxStylable *stylable,
  * Finds the #GParamSpec installed by @stylable for the property
  * with @property_name.
  *
- * Return value: a #GParamSpec for the given property, or %NULL if
- *   no property with that name was found
+ * Return value: (transfer none): a #GParamSpec for the given property,
+ *   or %NULL if no property with that name was found
  */
 GParamSpec *
 mx_stylable_find_property (MxStylable  *stylable,
@@ -419,7 +413,7 @@ mx_stylable_get_property_internal (MxStylable *stylable,
  * mx_stylable_get_property:
  * @stylable: a #MxStylable
  * @property_name: the name of the property
- * @value: return location for an empty #GValue
+ * @value: (out): return location for an empty #GValue
  *
  * Retrieves the value of @property_name for @stylable, and puts it
  * into @value.
@@ -520,7 +514,7 @@ mx_stylable_get (MxStylable  *stylable,
  * mx_stylable_get_default_value:
  * @stylable: a #MxStylable
  * @property_name: name of the property to query
- * @value_out: return location for the default value
+ * @value_out: (out): return location for the default value
  *
  * Query @stylable for the default value of property @property_name and
  * fill @value_out with the result.
@@ -565,7 +559,7 @@ mx_stylable_get_default_value (MxStylable  *stylable,
  * Retrieves the #MxStyle used by @stylable. This function does not
  * alter the reference count of the returned object.
  *
- * Return value: a #MxStyle
+ * Return value: (transfer none): a #MxStyle
  */
 MxStyle *
 mx_stylable_get_style (MxStylable *stylable)
@@ -624,18 +618,6 @@ mx_stylable_set_style (MxStylable *stylable,
   g_object_notify (G_OBJECT (stylable), "style");
 }
 
-/**
- * mx_stylable_changed:
- * @stylable: A #MxStylable
- *
- * Emit the "stylable-changed" signal on @stylable
- */
-void
-mx_stylable_changed (MxStylable *stylable)
-{
-  g_signal_emit (stylable, stylable_signals[CHANGED], 0, NULL);
-}
-
 /**
  * mx_stylable_get_style_pseudo_class:
  * @stylable: a #MxStylable
@@ -739,3 +721,164 @@ mx_stylable_set_style_class (MxStylable  *stylable,
                g_type_name (G_OBJECT_TYPE (stylable)));
 }
 
+static void
+mx_stylable_property_changed_notify (MxStylable *stylable)
+{
+  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_NONE);
+}
+
+static void
+mx_stylable_parent_set_notify (ClutterActor *actor,
+                               ClutterActor *old_parent)
+{
+  ClutterActor *new_parent = clutter_actor_get_parent (actor);
+
+  /* check the actor has a new parent */
+  if (new_parent)
+    {
+      mx_stylable_style_changed (MX_STYLABLE (actor), MX_STYLE_CHANGED_NONE);
+    }
+}
+
+static void mx_stylable_style_changed_internal (MxStylable          *stylable,
+                                                MxStyleChangedFlags  flags);
+
+static void
+mx_stylable_child_notify (ClutterActor *actor,
+                          gpointer      flags)
+{
+  if (MX_IS_STYLABLE (actor))
+    mx_stylable_style_changed_internal (MX_STYLABLE (actor),
+                                        GPOINTER_TO_INT (flags));
+}
+
+static void
+mx_stylable_style_changed_internal (MxStylable          *stylable,
+                                    MxStyleChangedFlags  flags)
+{
+
+  /* don't update stylables until they are mapped (unless ensure is set) */
+  if (!CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (stylable)) &&
+      !(flags & MX_STYLE_CHANGED_FORCE))
+    return;
+
+  g_signal_emit (stylable, stylable_signals[STYLE_CHANGED], 0, flags);
+
+  /* propagate the style-changed signal to children, since their style may
+   * depend on one or more properties of the parent */
+
+  if (CLUTTER_IS_CONTAINER (stylable))
+    {
+      /* notify our children that their parent stylable has changed */
+      clutter_container_foreach ((ClutterContainer *) stylable,
+                                 mx_stylable_child_notify,
+                                 GINT_TO_POINTER (flags));
+    }
+}
+
+/**
+ * mx_stylable_style_changed:
+ * @stylable: an MxStylable
+ * @flags: flags that control the style changing
+ *
+ * Emit the "style-changed" signal on @stylable to notify it that one or more
+ * of the style properties has changed.
+ *
+ * If @stylable is a #ClutterContainer then the "style-changed" notification is
+ * propagated to it's children, since their style may depend on one or more
+ * properties of the parent.
+ *
+ */
+void
+mx_stylable_style_changed (MxStylable *stylable, MxStyleChangedFlags flags)
+{
+  mx_stylable_style_changed_internal (stylable, flags);
+}
+
+void
+mx_stylable_connect_change_notifiers (MxStylable *stylable)
+{
+  g_return_if_fail (CLUTTER_IS_ACTOR (stylable));
+
+  g_return_if_fail (MX_IS_STYLABLE (stylable));
+
+  /* ClutterActor signals */
+  g_signal_connect (stylable, "notify::name",
+                    G_CALLBACK (mx_stylable_property_changed_notify), NULL);
+  g_signal_connect (stylable, "parent-set",
+                    G_CALLBACK (mx_stylable_parent_set_notify), NULL);
+
+  /* style-changed is blocked until the actor is mapped, so style-changed
+   * needs to be sent as soon as the actor is mapped */
+  g_signal_connect (stylable, "notify::mapped",
+                    G_CALLBACK (mx_stylable_property_changed_notify), NULL);
+
+  /* MxStylable notifiers */
+  g_signal_connect (stylable, "notify::style-class",
+                    G_CALLBACK (mx_stylable_property_changed_notify), NULL);
+  g_signal_connect (stylable, "notify::style-pseudo-class",
+                    G_CALLBACK (mx_stylable_property_changed_notify), NULL);
+}
+
+void
+mx_stylable_apply_clutter_text_attributes (MxStylable  *stylable,
+                                           ClutterText *text)
+{
+  ClutterColor *real_color = NULL;
+  gchar *font_name = NULL;
+  gint font_size = 0;
+  MxFontWeight font_weight;
+  PangoWeight weight;
+  PangoFontDescription *descr;
+  gchar *descr_string;
+
+  mx_stylable_get (stylable,
+                   "color", &real_color,
+                   "font-family", &font_name,
+                   "font-size", &font_size,
+                   "font-weight", &font_weight,
+                   NULL);
+
+
+  /* Create a description, we will convert to a string and set on the
+   * ClutterText. When Clutter gets API to set the description directly this
+   * won't be necessary. */
+  descr = pango_font_description_new ();
+
+  /* font name */
+  pango_font_description_set_family (descr, font_name);
+  g_free (font_name);
+
+  /* font size */
+  pango_font_description_set_absolute_size (descr, font_size * PANGO_SCALE);
+
+  /* font weight */
+  switch (font_weight)
+    {
+  case MX_FONT_WEIGHT_BOLD:
+    weight = PANGO_WEIGHT_BOLD;
+    break;
+  case MX_FONT_WEIGHT_LIGHTER:
+    weight = PANGO_WEIGHT_LIGHT;
+    break;
+  case MX_FONT_WEIGHT_BOLDER:
+    weight = PANGO_WEIGHT_HEAVY;
+    break;
+  default:
+    weight = PANGO_WEIGHT_NORMAL;
+    break;
+    }
+  pango_font_description_set_weight (descr, weight);
+
+  descr_string = pango_font_description_to_string (descr);
+  clutter_text_set_font_name (text, descr_string);
+  g_free (descr_string);
+  pango_font_description_free (descr);
+
+  /* font color */
+  if (real_color)
+    {
+      clutter_text_set_color (text, real_color);
+      clutter_color_free (real_color);
+    }
+}
