@@ -320,7 +320,7 @@ mx_toggle_button_release_event (ClutterActor       *actor,
 
   mx_toggle_set_active (toggle, !toggle->priv->active);
 
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -335,7 +335,7 @@ mx_toggle_handle_button_press_event (ClutterActor       *actor,
 
   toggle->priv->drag_offset = event->x;
 
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -464,6 +464,12 @@ mx_toggle_update_position (ClutterTimeline *timeline,
   clutter_actor_queue_relayout (CLUTTER_ACTOR (toggle));
 }
 
+static void
+mx_toggle_style_changed (MxToggle *toggle)
+{
+  mx_stylable_style_changed (MX_STYLABLE (toggle->priv->handle), 0);
+}
+
 static void
 mx_toggle_init (MxToggle *self)
 {
@@ -493,6 +499,9 @@ mx_toggle_init (MxToggle *self)
   g_signal_connect (self->priv->handle, "motion-event",
                     G_CALLBACK (mx_toggle_handle_motion_event), self);
 
+  g_signal_connect (self, "style-changed", G_CALLBACK (mx_toggle_style_changed),
+                    NULL);
+
 }
 
 ClutterActor *
