@@ -1,8 +1,9 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-viewport.c: Viewport actor
  *
  * Copyright 2008 OpenedHand
- * Copyright 2009 Intel Corporation.
+ * Copyright 2009, 2010 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -60,7 +61,7 @@
 #include "mx-private.h"
 #include "mx-bin.h"
 
-static void scrollable_interface_init (MxScrollableInterface *iface);
+static void scrollable_interface_init (MxScrollableIface *iface);
 
 static void scrollable_set_adjustments (MxScrollable *scrollable,
                                         MxAdjustment *hadjustment,
@@ -191,7 +192,7 @@ mx_viewport_set_property (GObject      *object,
       break;
 
     case PROP_SYNC_ADJUST:
-      priv->sync_adjustments = g_value_get_boolean (value);
+      mx_viewport_set_sync_adjustments (viewport, g_value_get_boolean (value));
       break;
 
     default:
@@ -220,42 +221,6 @@ mx_viewport_dispose (GObject *gobject)
   G_OBJECT_CLASS (mx_viewport_parent_class)->dispose (gobject);
 }
 
-static ClutterActor *
-get_child_and_natural_size (MxViewport *self,
-                            gfloat     *natural_width,
-                            gfloat     *natural_height)
-{
-  /* MxBin is a single-child container,
-   * let it grow as big as it wants. */
-  ClutterActor *child;
-  ClutterRequestMode mode;
-
-  child = mx_bin_get_child (MX_BIN (self));
-  if (child)
-    {
-
-      g_object_get (G_OBJECT (child), "request-mode", &mode, NULL);
-      if (mode == CLUTTER_REQUEST_HEIGHT_FOR_WIDTH)
-        {
-          clutter_actor_get_preferred_width (child, -1, NULL,
-                                             natural_width);
-          clutter_actor_get_preferred_height (child, *natural_width, NULL,
-                                              natural_height);
-        }
-      else
-        {
-          clutter_actor_get_preferred_height (child, -1, NULL,
-                                              natural_height);
-          clutter_actor_get_preferred_width (child, *natural_height, NULL,
-                                             natural_width);
-        }
-
-      return child;
-    }
-
-  return NULL;
-}
-
 static void
 mx_viewport_paint (ClutterActor *self)
 {
@@ -286,34 +251,31 @@ mx_viewport_allocate (ClutterActor          *self,
 {
   MxViewportPrivate *priv = MX_VIEWPORT (self)->priv;
   ClutterActor *child;
-  ClutterActorBox natural_box;
   gfloat natural_width, natural_height;
   gfloat available_width, available_height;
 
   /* Chain up. */
-  CLUTTER_ACTOR_CLASS (mx_viewport_parent_class)->
-  allocate (self, box, flags);
+  CLUTTER_ACTOR_CLASS (mx_viewport_parent_class)-> allocate (self, box, flags);
 
   available_width = box->x2 - box->x1;
   available_height = box->y2 - box->y1;
 
-  natural_box.x1 = 0;
-  natural_box.y1 = 0;
+  child = mx_bin_get_child (MX_BIN (self));
 
-  if (NULL != (child = get_child_and_natural_size (MX_VIEWPORT (self),
-                                                   &natural_width,
-                                                   &natural_height)))
+  if (child)
     {
-      natural_box.x2 = natural_width;
-      natural_box.y2 = natural_height;
-      clutter_actor_allocate (child, &natural_box, flags);
+      clutter_actor_allocate_preferred_size (child, flags);
+      clutter_actor_get_preferred_size (child, NULL, NULL, &natural_width,
+                                        &natural_height);
     }
   else
     {
-      natural_box.x2 = available_width;
-      natural_box.y2 = available_height;
+      natural_height = 0;
+      natural_width = 0;
     }
 
+
+
   /* Refresh adjustments */
   if (priv->sync_adjustments)
     {
@@ -326,6 +288,7 @@ mx_viewport_allocate (ClutterActor          *self,
                         "page-size", available_width,
                         "upper", natural_width,
                         "page-increment", available_width / 3,
+                        "step-increment", available_width / 12,
                         NULL);
 
           /* Make sure value is clamped */
@@ -340,6 +303,7 @@ mx_viewport_allocate (ClutterActor          *self,
                         "page-size", available_height,
                         "upper", natural_height,
                         "page-increment", available_height / 3,
+                        "step-increment", available_height / 12,
                         NULL);
 
           prev_value = mx_adjustment_get_value (priv->vadjustment);
@@ -395,9 +359,13 @@ mx_viewport_class_init (MxViewportClass *klass)
                                 MX_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_SYNC_ADJUST, pspec);
 
-  g_object_class_override_property (gobject_class, PROP_HADJUST, "hadjustment");
+  g_object_class_override_property (gobject_class,
+                                    PROP_HADJUST,
+                                    "horizontal-adjustment");
 
-  g_object_class_override_property (gobject_class, PROP_VADJUST, "vadjustment");
+  g_object_class_override_property (gobject_class,
+                                    PROP_VADJUST,
+                                    "vertical-adjustment");
 }
 
 static void
@@ -504,17 +472,14 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
           /* create an initial adjustment. this is filled with correct values
            * as soon as allocate() is called */
 
-          adjustment = mx_adjustment_new (0.0,
-                                          0.0,
-                                          1.0,
-                                          1.0,
-                                          1.0,
-                                          1.0);
+          adjustment = mx_adjustment_new ();
 
           scrollable_set_adjustments (scrollable,
                                       adjustment,
                                       priv->vadjustment);
 
+          g_object_unref (adjustment);
+
           *hadjustment = adjustment;
         }
     }
@@ -530,24 +495,21 @@ scrollable_get_adjustments (MxScrollable  *scrollable,
           /* create an initial adjustment. this is filled with correct values
            * as soon as allocate() is called */
 
-          adjustment = mx_adjustment_new (0.0,
-                                          0.0,
-                                          1.0,
-                                          1.0,
-                                          1.0,
-                                          1.0);
+          adjustment = mx_adjustment_new ();
 
           scrollable_set_adjustments (scrollable,
                                       priv->hadjustment,
                                       adjustment);
 
+          g_object_unref (adjustment);
+
           *vadjustment = adjustment;
         }
     }
 }
 
 static void
-scrollable_interface_init (MxScrollableInterface *iface)
+scrollable_interface_init (MxScrollableIface *iface)
 {
   iface->set_adjustments = scrollable_set_adjustments;
   iface->get_adjustments = scrollable_get_adjustments;
@@ -560,7 +522,8 @@ mx_viewport_init (MxViewport *self)
 
   self->priv->sync_adjustments = TRUE;
 
-  g_object_set (G_OBJECT (self), "reactive", FALSE, NULL);
+  g_object_set (G_OBJECT (self), "reactive", FALSE,
+                "clip-to-allocation", TRUE, NULL);
 }
 
 ClutterActor *
@@ -636,3 +599,25 @@ mx_viewport_get_origin (MxViewport *viewport,
     *z = priv->z;
 }
 
+void
+mx_viewport_set_sync_adjustments (MxViewport *viewport,
+                                  gboolean    sync)
+{
+  MxViewportPrivate *priv;
+
+  g_return_if_fail (MX_IS_VIEWPORT (viewport));
+
+  priv = viewport->priv;
+  if (priv->sync_adjustments != sync)
+    {
+      priv->sync_adjustments = sync;
+      g_object_notify (G_OBJECT (viewport), "sync-adjustments");
+    }
+}
+
+gboolean
+mx_viewport_get_sync_adjustments (MxViewport *viewport)
+{
+  g_return_val_if_fail (MX_IS_VIEWPORT (viewport), FALSE);
+  return viewport->priv->sync_adjustments;
+}
